@@ -347,7 +347,7 @@ Widget    new;
 
    else if (*name != NULL)
    {
-      if ((*widget = XwFindWidget (new -> core.parent, *name)) == NULL)
+      if ((*widget = XwFindWidget ((CompositeWidget)(new->core.parent), *name)) == NULL)
       {
 	 XtWarning ("Form: The reference widget was not found.");
          *name = new -> core.parent -> core.name;
