@@ -6,6 +6,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <signal.h>
 #include <sys/types.h>
 
 #ifdef HAVE_DIRENT_H
@@ -72,7 +73,7 @@ extern Tcl_Interp *xcinterp;
 #endif
 
 extern Display    *dpy;
-extern Clientdata areastruct;
+extern XCWindowData *areawin;
 extern ApplicationData appdata;
 extern int 	  *appcolors;
 extern short	  popups;     /* total number of popup windows */
@@ -117,7 +118,7 @@ void dragfilebox(xcWidget w, caddr_t clientdata, XMotionEvent *event)
    if (filenum == flcurrent) return;
 
    if (flcurrent >= 0) 		/* erase previous box */
-      XDrawRectangle(dpy, lwin, areastruct.gc, 5,
+      XDrawRectangle(dpy, lwin, areawin->gc, 5,
 	   10 + FILECHARHEIGHT * (flcurrent
 	   - flstart), flcurwidth + 10, FILECHARHEIGHT);
 
@@ -125,7 +126,7 @@ void dragfilebox(xcWidget w, caddr_t clientdata, XMotionEvent *event)
 
    twidth = XTextWidth(appdata.filefont, files[filenum].filename,
 	    strlen(files[filenum].filename));
-   XDrawRectangle(dpy, lwin, areastruct.gc, 5,
+   XDrawRectangle(dpy, lwin, areawin->gc, 5,
 	   10 + FILECHARHEIGHT * (filenum
 	   - flstart), twidth + 10, FILECHARHEIGHT);
 
@@ -146,17 +147,17 @@ void startfiletrack(xcWidget w, caddr_t clientdata, XCrossingEvent *event)
    xcAddEventHandler(w, PointerMotionMask, False, (xcEventHandler)dragfilebox, NULL);
 #endif
 
-   XSetFunction(dpy, areastruct.gc, GXcopy);
-   XSetForeground(dpy, areastruct.gc, AUXCOLOR);
-   XSetLineAttributes(dpy, areastruct.gc, 1, LineSolid, CapRound, JoinMiter);
+   XSetFunction(dpy, areawin->gc, GXcopy);
+   XSetForeground(dpy, areawin->gc, AUXCOLOR);
+   XSetLineAttributes(dpy, areawin->gc, 1, LineSolid, CapRound, JoinMiter);
 
    /* draw initial box */
 
    flcurrent = -1;
    dragfilebox(w, NULL, (XMotionEvent *)event);
 
-   XSetFunction(dpy, areastruct.gc, GXxor);
-   XSetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
+   XSetFunction(dpy, areawin->gc, GXxor);
+   XSetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
 }
 
 /*-------------------------------------------------------------------------*/
@@ -167,7 +168,7 @@ void endfiletrack(xcWidget w, caddr_t clientdata, XCrossingEvent *event)
 {
    Window lwin = xcWindow(w);
 
-   XDrawRectangle(dpy, lwin, areastruct.gc, 5,
+   XDrawRectangle(dpy, lwin, areawin->gc, 5,
 	   10 + FILECHARHEIGHT * (flcurrent
 	   - flstart), flcurwidth + 10, FILECHARHEIGHT);
 
@@ -180,8 +181,8 @@ void endfiletrack(xcWidget w, caddr_t clientdata, XCrossingEvent *event)
 #endif
 
    /* Restore graphics state values */
-   XSetForeground(dpy, areastruct.gc, areastruct.gccolor);
-   XSetFunction(dpy, areastruct.gc, areastruct.gctype);
+   XSetForeground(dpy, areawin->gc, areawin->gccolor);
+   XSetFunction(dpy, areawin->gc, areawin->gctype);
 }
 
 #endif
@@ -233,7 +234,7 @@ void crashrecover()
    }
    xobjs.tempfile = strdup(_STR2);
 
-   startloadfile();
+   startloadfile(-1);
 }
 
 /*----------------------------------------------------------------------*/
@@ -249,19 +250,41 @@ void findcrashfiles()
    uid_t userid = getuid();
 #endif
    time_t recent = 0;
+   char *snptr;
+   int pid;
 
    cwd = opendir(xobjs.tempdir);
    if (cwd == NULL) return;	/* No tmp directory, no tmp files! */
 
    while ((dp = readdir(cwd)) != NULL) {
       sprintf(_STR, "%s/%s", xobjs.tempdir, dp->d_name);
-      if (!strncmp(_STR + strlen(xobjs.tempdir) + 1, "XC", 2)) {
+      snptr = _STR + strlen(xobjs.tempdir) + 1;
+      if (!strncmp(snptr, "XC", 2)) {
+	 char *dotptr = strchr(snptr, '.');
+	 pid = -1;
+	 if (dotptr && dotptr > snptr + 2) {
+	    *dotptr = '\0';
+	    if (sscanf(snptr + 2, "%d", &pid) != 1) 
+	       pid = -1;
+	    *dotptr = '.';
+         }
          if ((!stat(_STR, &sbuf))
 #ifndef _MSC_VER
 	     && (sbuf.st_uid == userid)
 #endif
 	     ) {
 	    if ((recent == 0) || (sbuf.st_ctime > recent)) {
+
+	       /* Check if the PID belongs to an active process	*/
+	       /* by sending a CONT signal and checking if 	*/
+	       /* there was no error result.			*/
+
+#ifndef _MSC_VER
+	       if (pid != -1)
+		  if (kill((pid_t)pid, SIGCONT) == 0)
+		     continue;
+#endif
+
 	       recent = sbuf.st_ctime;
 	       strcpy(_STR2, _STR);
 	    }
@@ -279,12 +302,14 @@ void findcrashfiles()
 
 #ifdef TCL_WRAPPER
       char *cfile = getcrashfilename();
+
       sprintf(_STR, ".query.title.field configure -text "
 		"\"Recover file \'%s\'?\"", 
 		(cfile == NULL) ? "(unknown)" : cfile);
       Tcl_Eval(xcinterp, _STR);
       Tcl_Eval(xcinterp, ".query.bbar.okay configure -command "
 		"{filerecover; wm withdraw .query}; wm deiconify .query");
+      if (cfile != NULL) free(cfile);
 #else
       getfile(NULL, (pointertype)RECOVER, NULL);   /* Crash recovery mode */
 #endif
@@ -395,7 +420,7 @@ void listfiles(xcWidget w, popupstruct *okaystruct, caddr_t calldata)
 	    else if (match_filter(dp->d_name, filter))
 	       files[flfiles].filetype = MATCH;
 	    else {
-	       if (areastruct.filefilter)
+	       if (xobjs.filefilter)
 		  continue;
 	       else
 	          files[flfiles].filetype = NONMATCH;
@@ -424,7 +449,7 @@ void listfiles(xcWidget w, popupstruct *okaystruct, caddr_t calldata)
       pixheight = flfiles * FILECHARHEIGHT + 25;
       if (pixheight < textheight) pixheight = textheight;
 
-      flistpix = XCreatePixmap(dpy, areastruct.areawin, textwidth, pixheight,
+      flistpix = XCreatePixmap(dpy, areawin->window, textwidth, pixheight,
 	   DefaultDepthOfScreen(xcScreen(w)));
 
       /* Write the filenames onto the pixmap */
@@ -480,7 +505,7 @@ void newfilelist(xcWidget w, popupstruct *okaystruct)
       Wprintf("Error: Bad variable $XCOps(filter) in Tcl!");
       return;  
    }
-   areastruct.filefilter = bval;
+   xobjs.filefilter = bval;
 #endif
 
    for (n = 0; n < flfiles; n++)
@@ -711,7 +736,7 @@ void draglscroll(xcWidget w, popupstruct *okaystruct, XButtonEvent *event)
 
 void setfilefilter(xcWidget w, popupstruct *okaystruct, caddr_t calldata)
 {
-   areastruct.filefilter = (areastruct.filefilter) ? False : True;
+   xobjs.filefilter = (xobjs.filefilter) ? False : True;
 
    /* Force regeneration of the file list */
 
@@ -801,7 +826,7 @@ void genfilelist(xcWidget parent, popupstruct *okaystruct, Dimension width)
    wwidth = XTextWidth(appdata.xcfont, "filter", strlen("filter"));
    XtnSetArg(XtNx, width - wwidth - 50);
    XtnSetArg(XtNy, LISTHEIGHT + 10);
-   XtnSetArg(XtNset, areastruct.filefilter);
+   XtnSetArg(XtNset, xobjs.filefilter);
    XtnSetArg(XtNsquare, True);
    XtnSetArg(XtNborderWidth, 0);
    XtnSetArg(XtNfont, appdata.xcfont);
@@ -825,12 +850,12 @@ void genfilelist(xcWidget parent, popupstruct *okaystruct, Dimension width)
 /* Look for a directory name in a string and update cwdname accordingly	   */
 /*-------------------------------------------------------------------------*/
 
-int lookdirectory(char *lstring)
+int lookdirectory(char *lstring, int nchars)
 {
    int slen;
    DIR *cwd = NULL;
 
-   xc_tilde_expand(lstring);
+   xc_tilde_expand(lstring, nchars);
    slen = strlen(lstring);
 
    if (lstring[slen - 1] == '/' || ((cwd=opendir(lstring)) != NULL)) {
