@@ -145,9 +145,8 @@ FILE *findfontfile(char *fontname)
 	       }
 	       psname = (char *)malloc((1 + strlen(fontname)) * sizeof(char));
 	       strcpy(psname, fontname);
-	       sprintf(_STR, "No encoding file found for font %s: substituting %s",
+	       Wprintf("No encoding file found for font %s: substituting %s",
 			psname, fonts[fval].psname);
-               Wprintf(_STR);
                fonts = (fontinfo *)realloc(fonts, (fontcount + 1) * sizeof(fontinfo));
                fonts[fontcount].psname = psname;
                fonts[fontcount].family = psname;
@@ -176,7 +175,6 @@ FILE *findfontfile(char *fontname)
 /* Return 1 if successful, 0 if font is already present, and -1 if any	*/
 /* other error occurred.						*/
 /*----------------------------------------------------------------------*/
-extern Clientdata areastruct;
 
 int loadfontfile(char *fontname)
 {
