@@ -41,20 +41,15 @@
 /* Global Variable definitions						   */
 /*-------------------------------------------------------------------------*/
 
-extern short   eventmode;	/* keep track of the mode the screen is in */
 extern Display	*dpy;   /* Works well to make this globally accessible */
 extern int *appcolors;
 extern Cursor	appcursors[NUM_CURSORS];
 extern Globaldata xobjs;
-extern Clientdata areastruct;
+extern XCWindowData *areawin;
 extern char _STR[150];
 extern short fontcount;
 extern fontinfo *fonts;
 
-/*------------------------------------------------------*/
-/* Check if a library exists				*/
-/*-------------------------------------------------------*/
-
 /*---------------------------------------------------------*/
 /* Find the Helvetica font for use in labeling the objects */
 /*---------------------------------------------------------*/
@@ -113,12 +108,12 @@ int pageposition(short libmode, int x, int y, int mode)
 
    pages = (libmode == PAGELIB) ? xobjs.pages : xobjs.numlibs;
    computespacing(libmode, &gxsize, &gysize, &xdel, &ydel);
-   window_to_user(x, y, &areastruct.save);
+   window_to_user(x, y, &areawin->save);
 
    if (mode == 0) {	/* On-page */
-      if (areastruct.save.x >= 0 && areastruct.save.y <= 0) {
-         xin = areastruct.save.x / xdel;
-         yin = areastruct.save.y / ydel; 
+      if (areawin->save.x >= 0 && areawin->save.y <= 0) {
+         xin = areawin->save.x / xdel;
+         yin = areawin->save.y / ydel; 
          if (xin < gxsize && yin > -gysize) {
             bpage = (xin % gxsize) - (yin * gxsize);
             if (bpage < pages)
@@ -128,10 +123,10 @@ int pageposition(short libmode, int x, int y, int mode)
       return -1;
    }
    else {		/* Between-pages */
-      xin = (areastruct.save.x + (xdel >> 1)) / xdel;
+      xin = (areawin->save.x + (xdel >> 1)) / xdel;
       if (xin > gxsize) xin = gxsize;
       if (xin < 0) xin = 0;
-      yin = areastruct.save.y  / ydel; 
+      yin = areawin->save.y  / ydel; 
       if (yin > 0) yin = 0;
       if (yin < -gysize) yin = -gysize;
       bpage = (xin % (gxsize + 1)) + 1 - (yin * gxsize);
@@ -155,8 +150,10 @@ short pagelinks(int page)
    for (i = 0; i < xobjs.pages; i++)
       if (xobjs.pagelist[i]->pageinst != NULL)
 	 if (xobjs.pagelist[i]->pageinst->thisobject->parts > 0)
-	    if ((i == page) || (!strcmp(xobjs.pagelist[i]->filename,
-			xobjs.pagelist[page]->filename)))
+	    if ((i == page) || (xobjs.pagelist[i]->filename &&
+			xobjs.pagelist[page]->filename &&
+			(!strcmp(xobjs.pagelist[i]->filename,
+			xobjs.pagelist[page]->filename))))
 	       count++;
 
    return count;
@@ -216,9 +213,11 @@ short *pagetotals(int page, short mode)
 		  (*(counts + i)) = 1;
 	       else
 	       {
-	          if ((i == page) || (!strcmp(xobjs.pagelist[i]->filename,
-			xobjs.pagelist[page]->filename)))
-	             if (*(counts + i) == 0)
+	          if ((i == page) || (xobjs.pagelist[i]->filename
+			&& xobjs.pagelist[page]->filename
+			&& (!strcmp(xobjs.pagelist[i]->filename,
+			xobjs.pagelist[page]->filename))))
+		     if ((mode == INDEPENDENT) || (*(counts + i) == 0))
 		        (*(icount + i))++;
 	       }
 	    }
@@ -250,6 +249,23 @@ short *pagetotals(int page, short mode)
    }
 }
 
+/*---------------------------------------------------------*/
+/* Test whether a library instance is a "virtual" instance */
+/*---------------------------------------------------------*/
+
+Boolean is_virtual(objinstptr thisinst) {
+   int libno;
+   liblistptr ilist;
+
+   libno = libfindobject(thisinst->thisobject, NULL);
+
+   for (ilist = xobjs.userlibs[libno].instlist; ilist != NULL; ilist = ilist->next)
+      if ((ilist->thisinst == thisinst) && (ilist->virtual == TRUE))
+	 return TRUE;
+
+   return FALSE;
+}
+
 /*------------------------------------------------------*/
 /* Test whether an object is a page, and return the	*/
 /* page number if it is.  Otherwise, return -1.		*/
@@ -297,6 +313,121 @@ int check_library(char *libname)
    return -1;
 }
 
+/*------------------------------------------------------*/
+/* Check for library name (string).  Because XCircuit   */
+/* generates the text "Library: <filename>" for		*/
+/* library names, we also check against <filename>	*/
+/* only in these names.					*/
+/*------------------------------------------------------*/
+
+int name_to_library(char *libname)
+{
+   char *slib;
+   int i;
+
+   for (i = 0; i < xobjs.numlibs; i++) {
+      slib = xobjs.libtop[i + LIBRARY]->thisobject->name;
+      if (!strcmp(libname, slib)) {
+	 return i;
+      }
+      else if (!strncmp(slib, "Library: ", 9)) {
+	 if (!strcmp(libname, slib + 9)) {
+	    return i;
+         }
+      }
+   }
+   return -1;
+}
+
+/*------------------------------------------------------*/
+/* Move an object and all of its virtual instances from	*/
+/* one library to another.				*/
+/*------------------------------------------------------*/
+
+int libmoveobject(objectptr thisobject, int libtarget)
+{
+   int j, libsource;
+   liblistptr spec, slast, srch;
+
+   libsource = libfindobject(thisobject, &j);
+
+   if (libsource == libtarget) return libsource; /* nothing to do */
+   if (libsource < 0) return libsource;		 /* object not in the library */
+
+   /* Move the object from library "libsource" to library "libtarget" */
+
+   xobjs.userlibs[libtarget].library = (objectptr *)
+		realloc(xobjs.userlibs[libtarget].library,
+		(xobjs.userlibs[libtarget].number + 1) * sizeof(objectptr));
+
+   *(xobjs.userlibs[libtarget].library + xobjs.userlibs[libtarget].number) = thisobject;
+   xobjs.userlibs[libtarget].number++;
+
+   for (; j < xobjs.userlibs[libsource].number; j++)
+      *(xobjs.userlibs[libsource].library + j) =
+		*(xobjs.userlibs[libsource].library + j + 1);
+      xobjs.userlibs[libsource].number--;
+
+   /* Move all instances from library "libsource" to library "libtarget" */
+
+   slast = NULL;
+   for (spec = xobjs.userlibs[libsource].instlist; spec != NULL;) {
+      if (spec->thisinst->thisobject == thisobject) {
+
+	 /* Add to end of spec list in target */
+	 srch = xobjs.userlibs[libtarget].instlist;
+	 if (srch == NULL)
+	    xobjs.userlibs[libtarget].instlist = spec;
+	 else {
+	    for (; srch->next != NULL; srch = srch->next);
+	    spec->next = srch->next;
+	    srch->next = spec;
+	 }
+	 
+	 if (slast != NULL) {
+	    slast->next = spec->next;
+	    spec = slast->next;
+	 }
+	 else {
+	    xobjs.userlibs[libsource].instlist = spec->next;
+	    spec = xobjs.userlibs[libsource].instlist;
+	 }
+      }
+      else {
+	 slast = spec;
+	 spec = spec->next;
+      }
+   }
+
+   return libsource;
+}
+
+/*------------------------------------------------------*/
+/* Determine which library contains the specified	*/
+/* object.  If found, return the library number, or	*/
+/* else return -1 if the object was not found in any	*/
+/* library.  If "partidx" is non-null, fill with the	*/
+/* integer offset of the object from the beginning of	*/
+/* the library.						*/
+/*------------------------------------------------------*/
+
+int libfindobject(objectptr thisobject, int *partidx)
+{
+   int i, j;
+   objectptr libobj;
+
+   for (i = 0; i < xobjs.numlibs; i++) {
+      for (j = 0; j < xobjs.userlibs[i].number; j++) {
+         libobj = *(xobjs.userlibs[i].library + j);
+	 if (libobj == thisobject) {
+	    if (partidx != NULL) *partidx = j;
+	    return i;
+	 }
+      }
+   }
+   return -1;
+}
+
 /*------------------------------------------------------*/
 /* ButtonPress handler during page catalog viewing mode */
 /*------------------------------------------------------*/
@@ -307,7 +438,7 @@ void pagecat_op(int op, int x, int y)
    short mode;
 
    for (mode = 0; mode < LIBRARY; mode++) {
-      if (areastruct.topinstance == xobjs.libtop[mode]) break;
+      if (areawin->topinstance == xobjs.libtop[mode]) break;
    }
    if (mode == LIBRARY) return;  /* Something went wrong if this happens */
 
@@ -319,13 +450,13 @@ void pagecat_op(int op, int x, int y)
 	       /* using changepage() allows use of new page for schematic */
 	       changepage(bpage);
 	       /* associate the new schematic */
-	       schemassoc(topobject, areastruct.stack->thisinst->thisobject);
+	       schemassoc(topobject, areawin->stack->thisinst->thisobject);
 	       /* pop back to calling (symbol) page */
 	       catreturn();
 	       eventmode = NORMAL_MODE;
 	    }
 	    else {
-	       areastruct.lastlibrary = bpage;
+	       areawin->lastlibrary = bpage;
 	       startcatalog(NULL, (pointertype)(LIBRARY + bpage), NULL);
 	    }
 	    return;
@@ -422,8 +553,8 @@ void computespacing(short mode, int *gxsize, int *gysize, int *xdel, int *ydel)
 
    /* 0.5 is the default vscale;  g#size is the number of pages per line */
 
-   *xdel = areastruct.width / (0.5 * (*gxsize));
-   *ydel = areastruct.height / (0.5 * (*gysize));
+   *xdel = areawin->width / (0.5 * (*gxsize));
+   *ydel = areawin->height / (0.5 * (*gysize));
 }
 
 /*-------------------------------------------------------------------*/
@@ -576,23 +707,23 @@ void pagecatmove(int x, int y)
    objinstptr exchobj;
    Pagedata *ipage, **testpage, **tpage2;
 
-   if (areastruct.selects == 0) return;
-   else if (areastruct.selects > 2) {
+   if (areawin->selects == 0) return;
+   else if (areawin->selects > 2) {
       Wprintf("Select maximum of two objects.");
       return;
    }
 
    /* Get the page corresponding to the first selected object */
 
-   exchobj = SELTOOBJINST(areastruct.selectlist);
+   exchobj = SELTOOBJINST(areawin->selectlist);
    for (testpage = xobjs.pagelist; testpage < xobjs.pagelist + xobjs.pages; testpage++)
       if (*testpage != NULL && (*testpage)->pageinst == exchobj)
 	 break;
 
    /* If two objects are selected, then exchange their order */
 
-   if (areastruct.selects == 2) {
-      exchobj = SELTOOBJINST(areastruct.selectlist + 1);
+   if (areawin->selects == 2) {
+      exchobj = SELTOOBJINST(areawin->selectlist + 1);
       for (tpage2 = xobjs.pagelist; tpage2 < xobjs.pagelist + xobjs.pages; tpage2++)
 	 if (*tpage2 != NULL && (*tpage2)->pageinst == exchobj)
 	    break;
@@ -650,13 +781,14 @@ void composelib(short mode)
    labelptr *drawname;
    objectptr libobj, libpage = xobjs.libtop[mode]->thisobject;
    liblistptr spec;
-   int xpos = 0, ypos = areastruct.height << 1;
+   int xpos = 0, ypos = areawin->height << 1;
    int nypos = 220, nxpos;
    short fval;
    short llx, lly, width, height;
 
    int totalarea, targetwidth;
-   double scale;
+   double scale, savescale;
+   XPoint savepos;
 
    /* Also make composelib() a wrapper for composepagelib() */
    if ((mode > FONTLIB) && (mode < LIBRARY)) {
@@ -671,6 +803,11 @@ void composelib(short mode)
    for (pgen = libpage->plist; pgen < libpage->plist + libpage->parts; pgen++)
       if (IS_OBJINST(*pgen)) *pgen = NULL;
 
+   /* Before resetting, save the position and scale.  We will restore	*/
+   /* them at the end.							*/
+
+   savepos = libpage->pcorner;
+   savescale = libpage->viewscale;
    reset(libpage, NORMAL);
 
    /* Return if library defines no objects or virtual instances */
@@ -707,8 +844,8 @@ void composelib(short mode)
       totalarea += (width * height);
    }
 
-   scale = (double)totalarea / (double)(areastruct.width * areastruct.height);
-   targetwidth = (int)(sqrt(scale) * (double)areastruct.width);
+   scale = (double)totalarea / (double)(areawin->width * areawin->height);
+   targetwidth = (int)(sqrt(scale) * (double)areawin->width);
 
    /* generate the list of object instances and their labels */
 
@@ -731,7 +868,7 @@ void composelib(short mode)
       /* Determine the area needed on the page to draw the object */
 
       nxpos = xpos + ((width > 170) ? width + 30 : 200);
-      /* if ((nxpos > (areastruct.width << 1)) && (xpos > 0)) { */
+      /* if ((nxpos > (areawin->width << 1)) && (xpos > 0)) { */
       if ((nxpos > targetwidth) && (xpos > 0)) {
 	 nxpos -= xpos; 
 	 xpos = 0;
@@ -782,10 +919,13 @@ void composelib(short mode)
 
    /* Compute the bounding box of the library page */
    calcbbox(xobjs.libtop[mode]);
-   centerview(xobjs.libtop[mode]);
 
    /* Update the library directory */
    updatepagelib(LIBLIB, mode);
+
+   /* Restore original view position */
+   libpage->pcorner = savepos;
+   libpage->viewscale = savescale;
 }
 
 /*----------------------------------------------------------------*/
@@ -844,30 +984,22 @@ void catvirtualcopy()
    short i, *newselect;
    objinstptr libobj, libinst;
 
-   if (areastruct.selects == 0) return;
+   if (areawin->selects == 0) return;
    else if ((i = is_library(topobject)) < 0) return;
 
    /* Check for existance of parameters in the object for each */
    /* selected instance */
 
-   for (newselect = areastruct.selectlist; newselect < areastruct.selectlist
-	  + areastruct.selects; newselect++) {
+   for (newselect = areawin->selectlist; newselect < areawin->selectlist
+	  + areawin->selects; newselect++) {
       libobj = SELTOOBJINST(newselect);
-
-      if (libobj->thisobject->params == NULL) {
-	 Wprintf("Virtual copy allowed only on objects with paramters.");
-	 /* (because currently there's no way to adjust rotation/scale	*/
-	 /* of a non-parameterized instance from inside the library,	*/
-	 /* even though it can be specified in the library itself)	*/
-      }
-      else { 		/* All's clear to make a virtual copy */
-	 libinst = addtoinstlist(i, libobj->thisobject, TRUE);
-	 instcopy(libinst, libobj);
-      }
+      libinst = addtoinstlist(i, libobj->thisobject, TRUE);
+      instcopy(libinst, libobj);
    }
 
    clearselects();
    composelib(LIBRARY + i);
+   lib_mark_changed(i);
    drawarea(NULL, NULL, NULL);
 }
 
@@ -882,18 +1014,17 @@ void cathide()
    objectptr *compobj;
    objinstptr libobj;
 
-   if (areastruct.selects == 0) return;
+   if (areawin->selects == 0) return;
 
    /* Can only hide objects which are instances in other objects; */
    /* Otherwise, object would be "lost".			  */
 
-   for (newselect = areastruct.selectlist; newselect < areastruct.selectlist
-	  + areastruct.selects; newselect++) {
+   for (newselect = areawin->selectlist; newselect < areawin->selectlist
+	  + areawin->selects; newselect++) {
       libobj = SELTOOBJINST(newselect);
 
       if (finddepend(libobj, &compobj) == 0) {
-	 sprintf(_STR, "Cannot hide: no dependencies");
-	 Wprintf(_STR);
+	 Wprintf("Cannot hide: no dependencies");
       }
       else { 		/* All's clear to hide. */
 	 libobj->thisobject->hidden = True;
@@ -920,15 +1051,15 @@ void catdelete()
    liblistptr ilist, llist;
    objectptr *libpage, *compobj, *tlib, *slib;
 
-   if (areastruct.selects == 0) return;
+   if (areawin->selects == 0) return;
 
    if ((i = is_library(topobject)) >= 0) {
       libpage = xobjs.userlibs[i].library;
       libpobjs = &xobjs.userlibs[i].number;
    }
 
-   for (newselect = areastruct.selectlist; newselect < areastruct.selectlist
-	  + areastruct.selects; newselect++) {
+   for (newselect = areawin->selectlist; newselect < areawin->selectlist
+	  + areawin->selects; newselect++) {
       libobj = SELTOOBJINST(newselect);
 
       /* If this is just a "virtual copy", simply remove it from the list */
@@ -953,8 +1084,7 @@ void catdelete()
       /* or if the object is used on a page.				   */
 
       if (finddepend(libobj, &compobj)) {
-	 sprintf(_STR, "Cannot delete: dependency in \"%s\"", (*compobj)->name);
-	 Wprintf(_STR);
+	 Wprintf("Cannot delete: dependency in \"%s\"", (*compobj)->name);
       }
       else { 		/* All's clear to delete.		     	   */
 
@@ -1001,7 +1131,10 @@ void catdelete()
 
    clearselects();
 
-   if ((i = is_library(topobject)) >= 0) composelib(LIBRARY + i);
+   if ((i = is_library(topobject)) >= 0) {
+      composelib(LIBRARY + i);
+      lib_mark_changed(i);
+   }
 
    drawarea(NULL, NULL, NULL);
 }
@@ -1081,13 +1214,44 @@ void linkedlistinsertafter(liblistptr *spec, int o1, int o2)
    }
 }
 
+/*------------------------------------------------------*/
+/* Set the "changed" flag in a library if any object	*/
+/* in that library has changed.				*/
+/*------------------------------------------------------*/
+
+void lib_set_changes(libnum)
+{
+   int j;
+   objectptr thisobj;
+
+   if ((xobjs.userlibs[libnum].flags & LIBRARY_CHANGED) != 0)
+      return;
+
+   for (j = 0; j < xobjs.userlibs[libnum].number; j++) {
+      thisobj = *(xobjs.userlibs[libnum].library + j);
+      if (getchanges(thisobj) > 0) {
+	 xobjs.userlibs[libnum].flags |= LIBRARY_CHANGED;
+	 break;
+      }
+   }
+}
+
+/*------------------------------------------------------*/
+/* Mark a library as having been modified.		*/
+/*------------------------------------------------------*/
+
+void lib_mark_changed(int libnum)
+{
+   xobjs.userlibs[libnum].flags |= LIBRARY_CHANGED;
+}
+
 /*------------------------------------------------------*/
 /* Rearrange objects in the library			*/
 /*------------------------------------------------------*/
 
 void catmove(int x, int y)
 {
-   int i, j, s1, s2, ocentx, ocenty, rangex, rangey;
+   int i, j, k, s1, s2, ocentx, ocenty, rangex, rangey, l;
    liblistptr spec;
    objinstptr exchobj, lobj;
 
@@ -1098,80 +1262,90 @@ void catmove(int x, int y)
       return;
    }
 
-   if (areastruct.selects == 0) return;
-   else if (areastruct.selects > 2) {
-      Wprintf("Select maximum of two objects.");
-      return;
-   }
+   if (areawin->selects == 0) return;
 
-   exchobj = SELTOOBJINST(areastruct.selectlist);
-   s1 = -1;
+   /* Add selected object or objects at the cursor position */
+
+   window_to_user(x, y, &areawin->save);
+
+   s2 = -1;
    for (j = 0, spec = xobjs.userlibs[i].instlist; spec != NULL;
 		spec = spec->next, j++) {
-      if (spec->thisinst == exchobj) {
-	 s1 = j;
-	 break;
+      lobj = spec->thisinst;
+      for (k = 0; k < areawin->selects; k++) {
+	 exchobj = SELTOOBJINST(areawin->selectlist + k);
+	 if (lobj == exchobj) break;
       }
-   }
+      if (k < areawin->selects) continue;	/* ignore items being moved */
 
-   /* If two objects are selected, then exchange their order */
-
-   if (areastruct.selects == 2) {
-      lobj = SELTOOBJINST(areastruct.selectlist + 1);
-
-      s2 = -1;
-      for (j = 0, spec = xobjs.userlibs[i].instlist; spec != NULL;
-		spec = spec->next, j++) {
-	 if (spec->thisinst == lobj) {
-	    s2 = j;
-	    break;
-	 }
-      }
-      /* Exchange numbers s1 and s2 */
-      linkedlistswap(&(xobjs.userlibs[i].instlist), s1, s2);
-   }
-   else {	/* one object selected; find place to put from cursor position */
-
-      window_to_user(x, y, &areastruct.save);
-
-      s2 = -1;
-      for (j = 0, spec = xobjs.userlibs[i].instlist; spec != NULL;
-		spec = spec->next, j++) {
-	 lobj = spec->thisinst;
-
- 	 ocentx = lobj->position.x + lobj->bbox.lowerleft.x
+      ocentx = lobj->position.x + lobj->bbox.lowerleft.x
 	        + (lobj->bbox.width >> 1);
- 	 ocenty = lobj->position.y + lobj->bbox.lowerleft.y
+      ocenty = lobj->position.y + lobj->bbox.lowerleft.y
 	        + (lobj->bbox.height >> 1);
-	 rangey = (lobj->bbox.height > 200) ? 
+      rangey = (lobj->bbox.height > 200) ? 
 	        (lobj->bbox.height >> 1) : 100;
 
-	 if ((areastruct.save.y < ocenty + rangey) && (areastruct.save.y 
+      if ((areawin->save.y < ocenty + rangey) && (areawin->save.y 
 	         > ocenty - rangey)) {
-	    s2 = j - 1;
-	    if (areastruct.save.x < ocentx) break;
-	    else s2 = j;
-	 }
+	 s2 = j - 1;
+	 if (areawin->save.x < ocentx) break;
+	 else s2 = j;
       }
-      if ((s2 == -1) && (spec == NULL)) {
-	 if (areastruct.save.y <
+   }
+   if ((s2 == -1) && (spec == NULL)) {
+      if (areawin->save.y <
 		xobjs.libtop[i + LIBRARY]->thisobject->bbox.lowerleft.y)
-	    s2 = j - 1;
-	 else if (areastruct.save.y <=
+	 s2 = j - 1;
+      else if (areawin->save.y <=
 		xobjs.libtop[i + LIBRARY]->thisobject->bbox.lowerleft.y +
 		xobjs.libtop[i + LIBRARY]->thisobject->bbox.height) {
-	    unselect_all();	
-	    Wprintf("Could not find appropriate place to insert object");
-	    return;
-	 }
+	 unselect_all();	
+	 Wprintf("Could not find appropriate place to insert object");
+	 return;
       }
+   }
 
-      /* Move number s1, insert it after s2 */
-      linkedlistinsertafter(&(xobjs.userlibs[i].instlist), s1, s2);
+   /* Find object number s2 (because s2 value may change during insertion) */
+   if (s2 > -1) {
+      for (k = 0, spec = xobjs.userlibs[i].instlist; k < s2; spec = spec->next, k++);
+      lobj = spec->thisinst;
+   }
+   else lobj = NULL;
+
+   /* Move items; insert them after item s2 in order selected */
+
+   j = i;
+   for (k = 0; k < areawin->selects; k++) {
+
+      /* Find number of lobj (may have changed) */
+      for (s2 = 0, spec = xobjs.userlibs[i].instlist; spec != NULL;
+		spec = spec->next, s2++)
+	 if (spec->thisinst == lobj)
+	    break;
+
+      exchobj = SELTOOBJINST(areawin->selectlist + k);
+      for (s1 = 0, spec = xobjs.userlibs[i].instlist; spec != NULL;
+		spec = spec->next, s1++)
+         if (spec->thisinst == exchobj)
+	    break;
+
+      if (spec == NULL) {
+	 /* Object came from another library */
+         if ((l = libmoveobject(exchobj->thisobject, i)) >= 0) j = l;
+      }
+      else {
+         linkedlistinsertafter(&(xobjs.userlibs[i].instlist), s1, s2);
+      }
    }
 
    unselect_all();
-   if ((i = is_library(topobject)) >= 0) composelib(LIBRARY + i);
+   composelib(LIBRARY + i);
+   lib_mark_changed(i);
+   if (j != i) {
+      composelib(LIBRARY + j);
+      centerview(xobjs.libtop[LIBRARY + j]);
+      lib_mark_changed(j);
+   }
 
    drawarea(NULL, NULL, NULL);
 }
@@ -1188,8 +1362,8 @@ void copycat()
    oparamptr ops, newops;
    int i, libnum = USERLIB - LIBRARY ;
 
-   for (newselect = areastruct.selectlist; newselect < areastruct.selectlist
-	  + areastruct.selects; newselect++) {
+   for (newselect = areawin->selectlist; newselect < areawin->selectlist
+	  + areawin->selects; newselect++) {
 
       libobj = SELTOOBJINST(newselect);
       oldobj = libobj->thisobject;
@@ -1349,9 +1523,10 @@ void copycat()
    addtoinstlist(USERLIB - LIBRARY, *newobj, FALSE);
 
    composelib(USERLIB);
+   lib_mark_changed(USERLIB - LIBRARY);
    unselect_all();
 
-   if (areastruct.topinstance == xobjs.libtop[USERLIB])
+   if (areawin->topinstance == xobjs.libtop[USERLIB])
       drawarea(NULL, NULL, NULL);
    else startcatalog(NULL, (pointertype)USERLIB, NULL);
 }
@@ -1383,7 +1558,7 @@ void catalog_op(int op, int x, int y)
    }
    else {
 
-      window_to_user(x, y, &areastruct.save);
+      window_to_user(x, y, &areawin->save);
 	
       for (libobj = (objinstptr *)topobject->plist; libobj <
 		(objinstptr *)topobject->plist + topobject->parts; libobj++) {
@@ -1399,18 +1574,20 @@ void catalog_op(int op, int x, int y)
 	    rangey = ((*libobj)->bbox.height > 200) ? 
 	        ((*libobj)->bbox.height >> 1) : 100;
 
-	    if (areastruct.save.x > ocentx - rangex && areastruct.save.x <
-		   ocentx + rangex && areastruct.save.y < ocenty + rangey
-		   && areastruct.save.y > ocenty - rangey) {
+	    if (areawin->save.x > ocentx - rangex && areawin->save.x <
+		   ocentx + rangex && areawin->save.y < ocenty + rangey
+		   && areawin->save.y > ocenty - rangey) {
 
 	       /* setup to move object around and draw the selected object */
 
 	       if (eventmode == ASSOC_MODE) {
 
 	          /* revert to old page */
-	          areastruct.topinstance = (areastruct.stack == NULL) ?
-			   xobjs.pagelist[areastruct.page]->pageinst
-			   : areastruct.stack->thisinst;
+		  topobject->viewscale = areawin->vscale;
+		  topobject->pcorner = areawin->pcorner;
+	          areawin->topinstance = (areawin->stack == NULL) ?
+			   xobjs.pagelist[areawin->page]->pageinst
+			   : areawin->stack->thisinst;
 		  /* associate the new object */
 		  schemassoc(topobject, (*libobj)->thisobject); 
    	          setpage(TRUE);
@@ -1421,9 +1598,11 @@ void catalog_op(int op, int x, int y)
 
 	          /* revert to old page */
 
-	          areastruct.topinstance = (areastruct.stack == NULL) ?
-			   xobjs.pagelist[areastruct.page]->pageinst
-			   : areastruct.stack->thisinst;
+		  topobject->viewscale = areawin->vscale;
+		  topobject->pcorner = areawin->pcorner;
+	          areawin->topinstance = (areawin->stack == NULL) ?
+			   xobjs.pagelist[areawin->page]->pageinst
+			   : areawin->stack->thisinst;
 
 	          /* retrieve drawing window state and set position of object to
 	             be correct in reference to that window */
@@ -1432,19 +1611,19 @@ void catalog_op(int op, int x, int y)
 
    	          setpage(FALSE);
 	    
-	          snap(x, y, &areastruct.save);
-	          xdiff = areastruct.save.x - oldpos.x;
-	          ydiff = areastruct.save.y - oldpos.y;
+	          snap(x, y, &areawin->save);
+	          xdiff = areawin->save.x - oldpos.x;
+	          ydiff = areawin->save.y - oldpos.y;
 
                   /* collect all of the selected items */
 
-	          for (newselect = areastruct.selectlist; newselect <
-		     areastruct.selectlist + areastruct.selects; newselect++) {
+	          for (newselect = areawin->selectlist; newselect <
+		     areawin->selectlist + areawin->selects; newselect++) {
 		     NEW_OBJINST(newobject, topobject);
 		     topobject->parts++;
 		     instcopy(*newobject, TOOBJINST(libpage->plist + *newselect));
 		     /* color should be recast as current color */
-		     (*newobject)->color = areastruct.color;
+		     (*newobject)->color = areawin->color;
 		     /* position modified by (xdiff, ydiff) */
 		     (*newobject)->position.x += xdiff;
 		     (*newobject)->position.y += ydiff;
@@ -1461,9 +1640,9 @@ void catalog_op(int op, int x, int y)
 		     NEW_OBJINST(newobject, topobject);
                      topobject->parts++;
 		     instcopy(*newobject, *libobj);
-		     (*newobject)->color = areastruct.color;
-                     (*newobject)->position.x = areastruct.save.x;
-	             (*newobject)->position.y = areastruct.save.y; 
+		     (*newobject)->color = areawin->color;
+                     (*newobject)->position.x = areawin->save.x;
+	             (*newobject)->position.y = areawin->save.y; 
 
 	             /* add this object to the list of selected items */
 
@@ -1475,25 +1654,25 @@ void catalog_op(int op, int x, int y)
 
 		     /* Key "c" pressed for "copy" (default binding) */
 
-                     XDefineCursor(dpy, areastruct.areawin, COPYCURSOR);
+                     XDefineCursor(dpy, areawin->window, COPYCURSOR);
                      eventmode = COPY_MODE;
 #ifndef TCL_WRAPPER
-                     xcAddEventHandler(areastruct.area, PointerMotionMask, False,
+                     xcAddEventHandler(areawin->area, PointerMotionMask, False,
 			  (xcEventHandler)xlib_drag, NULL);
 #endif
 	          }
 	          else {
                      eventmode = MOVE_MODE;
-	             areastruct.editcycle = 1;
+	             areawin->editcycle = 1;
 	          }
 #ifdef TCL_WRAPPER
 		  /* fprintf(stderr, "Creating event handler for xctk_drag: "); */
 		  /* printeventmode();		*/
-                  Tk_CreateEventHandler(areastruct.area, PointerMotionMask,
+                  Tk_CreateEventHandler(areawin->area, PointerMotionMask,
 			(Tk_EventProc *)xctk_drag, NULL);
 #endif
-	 	  register_for_undo(XCF_Library_Pop, UNDO_MORE, areastruct.topinstance,
-				areastruct.selectlist, areastruct.selects);
+	 	  register_for_undo(XCF_Library_Pop, UNDO_MORE, areawin->topinstance,
+				areawin->selectlist, areawin->selects);
                   catreturn();
                }
 
@@ -1504,10 +1683,10 @@ void catalog_op(int op, int x, int y)
 	       else if (op == XCF_Select) {
 		  short newinst = (short)(libobj - (objinstptr *)topobject->plist);
 		  /* (ignore this object if it is already in the list of selects) */
-		  for (newselect = areastruct.selectlist; newselect <
-			areastruct.selectlist + areastruct.selects; newselect++)
+		  for (newselect = areawin->selectlist; newselect <
+			areawin->selectlist + areawin->selects; newselect++)
 		     if (*newselect == newinst) break;
-		  if (newselect == areastruct.selectlist + areastruct.selects) {
+		  if (newselect == areawin->selectlist + areawin->selects) {
 		     newselect = allocselect();
 		     *newselect = newinst;
 		     XcSetFunction(GXcopy);
@@ -1531,8 +1710,8 @@ void changecat()
    int i, j;
 
    if ((i = is_library(topobject)) < 0) {
-      if (areastruct.lastlibrary >= xobjs.numlibs) areastruct.lastlibrary = 0;
-      j = areastruct.lastlibrary;
+      if (areawin->lastlibrary >= xobjs.numlibs) areawin->lastlibrary = 0;
+      j = areawin->lastlibrary;
       eventmode = CATALOG_MODE;
    }
    else {
@@ -1541,8 +1720,12 @@ void changecat()
 	 Wprintf("This is the only library.");
 	 return;
       }
-      areastruct.lastlibrary = j;
+      areawin->lastlibrary = j;
    }
+
+   if (eventmode == CATMOVE_MODE)
+      delete_for_xfer(NORMAL, areawin->selectlist, areawin->selects);
+
    startcatalog(NULL, (pointertype)(j + LIBRARY), NULL);
 }
 
@@ -1555,23 +1738,23 @@ void startcatalog(xcWidget w, pointertype libmod, caddr_t nulldata)
    if (xobjs.libtop == NULL) return;	/* No libraries defined */
 
    if ((xobjs.libtop[libmod]->thisobject == NULL) ||
-		(areastruct.topinstance == xobjs.libtop[libmod])) return;
+		(areawin->topinstance == xobjs.libtop[libmod])) return;
 
    if (libmod == FONTLIB) {
-      XDefineCursor (dpy, areastruct.areawin, DEFAULTCURSOR);
+      XDefineCursor (dpy, areawin->window, DEFAULTCURSOR);
       if (eventmode == TEXT_MODE)
          eventmode = FONTCAT_MODE;
       else
          eventmode = EFONTCAT_MODE;
    }
    else if (eventmode == ASSOC_MODE) {
-      XDefineCursor (dpy, areastruct.areawin, DEFAULTCURSOR);
+      XDefineCursor (dpy, areawin->window, DEFAULTCURSOR);
    }
    else if (libmod == PAGELIB || libmod == LIBLIB) {
-      XDefineCursor (dpy, areastruct.areawin, DEFAULTCURSOR);
+      XDefineCursor (dpy, areawin->window, DEFAULTCURSOR);
       eventmode = CATALOG_MODE;
    }
-   else
+   else if (eventmode != CATMOVE_MODE)
       eventmode = CATALOG_MODE;
 
    /* Push the current page onto the push stack, unless	we're going */
@@ -1580,18 +1763,22 @@ void startcatalog(xcWidget w, pointertype libmod, caddr_t nulldata)
 
 
    if (!(((is_library(topobject) >= 0)
-		|| (areastruct.topinstance == xobjs.libtop[LIBLIB])
-		|| (areastruct.topinstance == xobjs.libtop[PAGELIB]))
+		|| (areawin->topinstance == xobjs.libtop[LIBLIB])
+		|| (areawin->topinstance == xobjs.libtop[PAGELIB]))
 		&& libmod >= PAGELIB)) {
-      push_stack(&areastruct.stack, areastruct.topinstance);
+      push_stack(&areawin->stack, areawin->topinstance);
    }
 
    /* set library as new object */
 
-   areastruct.topinstance = xobjs.libtop[libmod];
+   topobject->viewscale = areawin->vscale;
+   topobject->pcorner = areawin->pcorner;
+   areawin->topinstance = xobjs.libtop[libmod];
    setpage(TRUE);
    invalidate_graphics(topobject);
 
+   transferselects();
+
    /* draw the new screen */
 
    refresh(NULL, NULL, NULL);
