@@ -16,6 +16,9 @@ extern void free_undo_record(Undoptr);
 extern void free_redo_record(Undoptr);
 extern stringpart *get_original_string(labelptr);
 extern short *recover_selectlist(Undoptr);
+extern void free_selection(uselection *);
+extern uselection *remember_selection(objinstptr, short *, int);
+extern short *regen_selection(objinstptr, uselection *);
 #ifndef TCL_WRAPPER
 extern void undo_call(xcWidget, caddr_t, caddr_t);
 extern void redo_call(xcWidget, caddr_t, caddr_t);
@@ -31,6 +34,8 @@ extern Tk_EventProc *xctk_dragfilebox(ClientData, XEvent *);
 extern int XcTagCallback(Tcl_Interp *, int, Tcl_Obj *CONST objv[]);
 extern Tcl_Obj *evaluate_raw(oparamptr, int *);
 extern char *TCL_to_PS(char *);
+extern XCWindowData *GUI_init(int, Tcl_Obj *CONST objv[]);
+extern void build_app_database(Tk_Window);
 #endif
 
 extern char *evaluate_expr(oparamptr, objinstptr);
@@ -110,6 +115,7 @@ extern void wire_op(int, int ,int);
 
 extern Boolean recursefind(objectptr, objectptr);
 extern void transferselects();
+extern void select_invalidate_netlist();
 extern void newmatrix();
 extern void setpage(Boolean);
 extern int changepage(short);
@@ -143,7 +149,8 @@ extern void nextarccycle(arcptr, short);
 extern void buttonhandler(xcWidget, caddr_t, XButtonEvent *);
 extern void keyhandler(xcWidget, caddr_t, XKeyEvent *);
 extern Boolean compatible_function(int);
-extern void eventdispatch(int, int, int);
+extern int eventdispatch(int, int, int);
+extern int functiondispatch(int, short, int, int);
 extern void releasehandler(xcWidget, caddr_t, XKeyEvent *);
 extern void setsnap(short);
 extern void snapelement();
@@ -157,17 +164,17 @@ extern XPoint *pathclosepoint(pathptr, XPoint *);
 extern void placeselects(short, short, XPoint *);
 extern void drag(int, int);
 extern void xlib_drag(xcWidget, caddr_t, XEvent *);
-extern void elemrotate(genericptr *, short);
-extern void elementrotate(short);
+extern void elemrotate(genericptr *, short, XPoint *);
+extern void elementrotate(short, XPoint *);
 extern void edit(int, int);
 extern void pathedit(genericptr *, short);
 extern void xc_lower(short *);
 extern void xc_raise(short *);
 extern void exchange();
-extern void elhflip(genericptr *);
-extern void elvflip(genericptr *);
-extern void elementflip();
-extern void elementvflip();
+extern void elhflip(genericptr *, short);
+extern void elvflip(genericptr *, short);
+extern void elementflip(XPoint *);
+extern void elementvflip(XPoint *);
 #ifdef TCL_WRAPPER
 extern xcTimeOutProc makepress(caddr_t);
 #else
@@ -222,7 +229,7 @@ extern void fileselect(xcWidget, popupstruct *, XButtonEvent *);
 extern void showlscroll(xcWidget, caddr_t, caddr_t);
 extern void draglscroll(xcWidget, popupstruct *, XButtonEvent *);
 extern void genfilelist(xcWidget, popupstruct *, Dimension);
-extern int lookdirectory(char *);
+extern int lookdirectory(char *, int);
 
 /* from files.c: */
 
@@ -254,10 +261,11 @@ extern void loadulib();
 extern void loadblib();
 extern void getlib(xcWidget, caddr_t, caddr_t);
 extern void getuserlib(xcWidget, caddr_t, caddr_t);
-extern void loadlibrary(short);
-extern void startloadfile();
+extern Boolean loadlibrary(short);
+extern void startloadfile(int);
+extern void normalloadfile();
 extern void importfile();
-extern void loadfile(short);
+extern Boolean loadfile(short, int);
 extern void readlabel(objectptr, char *, stringpart **);
 extern void readparams(objectptr, objinstptr, objectptr, char *);
 extern u_char *find_match(u_char *);
@@ -287,7 +295,9 @@ extern void findfonts(objectptr, short *);
 extern void savefile(short);
 extern int printRGBvalues(char *, int, const char *);
 extern char *nosprint(char *);
-extern Boolean xc_tilde_expand(char *);
+extern FILE *fileopen(char *, char *, char *, int);
+extern Boolean xc_tilde_expand(char *, int);
+extern Boolean xc_variable_expand(char *, int);
 extern short writelabel(FILE *, stringpart *, short *);
 extern char *writesegment(stringpart *, float *, int *);
 extern int writelabelsegs(FILE *, short *, stringpart *);
@@ -401,23 +411,23 @@ extern void printhelp(xcWidget);
 
 /* from keybindings.c */
 
-extern int firstbinding(int);
-extern Boolean ismacro(int);
-extern Boolean isbound(int, int);
-extern Boolean isnbound(int, int, short *);
-extern int boundfunction(int, int);
+extern int firstbinding(xcWidget, int);
+extern Boolean ismacro(xcWidget, int);
+extern int boundfunction(xcWidget, int, short *);
 extern int string_to_func(const char *, short *);
 extern int string_to_key(const char *);
-extern char *binding_to_string(int, int);
+extern char *function_binding_to_string(xcWidget, int);
+extern char *key_binding_to_string(xcWidget, int);
+extern char *compat_key_to_string(xcWidget, int);
 extern char *func_to_string(int);
 extern char *key_to_string(int);
 extern void printeditbindings();
-extern int add_vbinding(int, int, short);
-extern int add_binding(int, int);
-extern int add_keybinding(const char *, const char *);
+extern int add_vbinding(xcWidget, int, int, short);
+extern int add_binding(xcWidget, int, int);
+extern int add_keybinding(xcWidget, const char *, const char *);
 extern void default_keybindings();
-extern int remove_binding(int, int);
-extern void remove_keybinding(const char *, const char *);
+extern int remove_binding(xcWidget, int, int);
+extern void remove_keybinding(xcWidget, const char *, const char *);
 
 #ifndef TCL_WRAPPER
 extern void mode_rebinding(int, int);
@@ -431,8 +441,14 @@ extern void catreturn();
 extern int pageposition(short, int, int, int);
 extern short pagelinks();
 extern short *pagetotals(int, short);
+extern Boolean is_virtual(objinstptr);
 extern int is_page(objectptr);
 extern int is_library(objectptr);
+extern int name_to_library(char *);
+extern void lib_set_changes();
+extern void lib_mark_changed(int);
+extern int libfindobject(objectptr, int *);
+extern int libmoveobject(objectptr, int);
 extern void pagecat_op(int, int, int);
 extern void pageinstpos(short, short, objinstptr, int, int, int, int);
 extern objinstptr newpageinst(objectptr);
@@ -453,85 +469,32 @@ extern void startcatalog(xcWidget, pointertype, caddr_t);
 
 /* from menucalls.c: */
 
-extern void dotoolbar(xcWidget, caddr_t, caddr_t);
-extern void overdrawpixmap(xcWidget);
-extern void getgeneric(buttonsave *, xcWidget, void (*getfunction)(), void *);
 extern void setgrid(xcWidget, float *);
 extern void measurestr(float, char *);
-extern void getsnapspace(xcWidget, caddr_t, caddr_t);
-extern void getgridspace(xcWidget, caddr_t, caddr_t);
-extern void setfloat(xcWidget, float *);
 extern void setwidth(xcWidget, float *);
-extern void settsize(xcWidget, labelptr);
-extern void autoset(xcWidget, xcWidgetList, caddr_t);
-extern void autostop(xcWidget, caddr_t, caddr_t);
+extern void changetextscale(float);
 extern void autoscale(int);
-extern void setscaley(xcWidget, float *);
-extern void setscalex(xcWidget, float *);
-extern void setorient(xcWidget, short *);
-extern void setpmode(xcWidget, short *);
-extern void setpagesize(xcWidget, XPoint *);
-extern void getkern(xcWidget, caddr_t, caddr_t);
-extern void setkern(xcWidget, stringpart *);
-extern void getdscale(xcWidget, caddr_t, caddr_t);
-extern void setdscale(xcWidget, XPoint *);
-extern labelptr gettextsize(float **);
-extern void gettsize(xcWidget, caddr_t, caddr_t);
-extern void setosize(xcWidget, objinstptr);
-extern void getosize(xcWidget, caddr_t, caddr_t);
-extern void getwirewidth(xcWidget, caddr_t, caddr_t);
-extern void setwwidth(xcWidget, void *);
-extern void getwwidth(xcWidget, caddr_t, caddr_t);
-extern void getfloat(xcWidget, float *, caddr_t);
-extern void setfilename(xcWidget, char **);
-extern void setpagelabel(xcWidget, char *);
-extern void makenewfontbutton();
-extern void makenewencodingbutton(char *, char);
-extern void togglefontmark(int);
-#ifndef TCL_WRAPPER
-extern void toggleexcl(xcWidget, menuptr, int);
-extern void highlightexcl(xcWidget, int, int);
-extern void toolcursor(int);
-extern void promptparam(xcWidget, caddr_t, caddr_t);
-#endif
-extern void stringparam(xcWidget, caddr_t, caddr_t);
-extern void toggle(xcWidget, Boolean *, caddr_t);
-extern int setelementstyle(xcWidget, u_short, u_short);
-extern void setcolorscheme(Boolean);
-extern void inversecolor(xcWidget, Boolean *, caddr_t);
-extern void togglegrid(u_short);
 extern float parseunits(char *);
-extern void setgridtype(char *);
+extern Boolean setoutputpagesize(XPoint *);
+extern void setkern(xcWidget, stringpart *);
+extern void setdscale(xcWidget, XPoint *);
+extern void setosize(xcWidget, objinstptr);
+extern void setwwidth(xcWidget, void *);
+extern labelptr gettextsize(float **);
+extern void stringparam(xcWidget, caddr_t, caddr_t);
+extern int setelementstyle(xcWidget, u_short, u_short);
+extern void togglegrid(u_short);
+extern void togglefontmark(int);
+extern void setcolorscheme(Boolean);
 extern void getgridtype(xcWidget, pointertype, caddr_t);
 extern void newlibrary(xcWidget, caddr_t, caddr_t);
-extern int findemptylib();
 extern int createlibrary(Boolean);
-extern void newpagemenu(xcWidget, pointertype, caddr_t);
 extern void makepagebutton();
-extern void renamepage(short);
-extern void renamelib(short);
-extern void setcolormark(int);
-extern void setallstylemarks(u_short);
+extern int findemptylib();
 extern polyptr checkforbbox(objectptr);
-#ifndef TCL_WRAPPER
-extern void setfill(xcWidget, pointertype, caddr_t);
-extern void makebbox(xcWidget, pointertype, caddr_t);
-extern void setclosure(xcWidget, pointertype, caddr_t);
-extern void setopaque(xcWidget, pointertype, caddr_t);
-extern void changetool(xcWidget, pointertype, caddr_t);
-extern void exec_or_changetool(xcWidget, pointertype, caddr_t);
-extern void rotatetool(xcWidget, pointertype, caddr_t);
-extern void pantool(xcWidget, pointertype, caddr_t);
-#endif
-extern int setline(xcWidget, pointertype, caddr_t);
 #ifdef TCL_WRAPPER
 extern void setcolor(xcWidget, int);
-#else
-extern void setcolor(xcWidget, pointertype, caddr_t);
 #endif
-extern void setnewcolor(xcWidget, caddr_t);
-extern void addnewcolor(xcWidget, caddr_t, caddr_t);
-extern void setfontmarks(short, short);
 extern void startparam(xcWidget, pointertype, caddr_t);
 extern void startunparam(xcWidget, pointertype, caddr_t);
 extern void setdefaultfontmarks();
@@ -542,7 +505,6 @@ extern void setvjust(xcWidget, pointertype, caddr_t);
 extern void sethjust(xcWidget, pointertype, caddr_t);
 extern void boxedit(xcWidget, pointertype, caddr_t);
 extern void locloadfont(xcWidget, char *);
-extern void addnewfont(xcWidget, caddr_t, caddr_t);
 extern short findbestfont(short, short, short, short);
 extern void setfontval(xcWidget, pointertype, labelptr);
 extern void setfont(xcWidget, pointertype, caddr_t);
@@ -551,12 +513,72 @@ extern void fontstyle(xcWidget, pointertype, caddr_t);
 extern void setfontencoding(xcWidget, pointertype, labelptr);
 extern void fontencoding(xcWidget, pointertype, caddr_t);
 extern void addtotext(xcWidget, pointertype, caddr_t);
+extern Boolean dospecial(void);
+
+/* from xtfuncs.c: */
+
+#ifndef TCL_WRAPPER
+extern void setfloat(xcWidget, float *);
+extern void autoset(xcWidget, xcWidgetList, caddr_t);
+extern void autostop(xcWidget, caddr_t, caddr_t);
+extern void togglegridstyles(xcWidget);
+extern void togglejustifs(xcWidget);
+extern void togglefontstyles(xcWidget);
+extern void toggleencodings(xcWidget);
+extern void getkern(xcWidget, caddr_t, caddr_t);
+extern void setcolor(xcWidget, pointertype, caddr_t);
+extern void setfill(xcWidget, pointertype, caddr_t);
+extern void makebbox(xcWidget, pointertype, caddr_t);
+extern void setclosure(xcWidget, pointertype, caddr_t);
+extern void setopaque(xcWidget, pointertype, caddr_t);
+extern void setline(xcWidget, pointertype, caddr_t);
+extern void changetool(xcWidget, pointertype, caddr_t);
+extern void exec_or_changetool(xcWidget, pointertype, caddr_t);
+extern void rotatetool(xcWidget, pointertype, caddr_t);
+extern void pantool(xcWidget, pointertype, caddr_t);
+extern void toggleexcl(xcWidget, menuptr, int);
+extern void highlightexcl(xcWidget, int, int);
+extern void toolcursor(int);
+extern void promptparam(xcWidget, caddr_t, caddr_t);
+extern void gettsize(xcWidget, caddr_t, caddr_t);
+extern void settsize(xcWidget, labelptr);
+extern void dotoolbar(xcWidget, caddr_t, caddr_t);
+extern void overdrawpixmap(xcWidget);
+extern buttonsave *getgeneric(xcWidget, void (*getfunction)(), void *);
+extern void getsnapspace(xcWidget, caddr_t, caddr_t);
+extern void getgridspace(xcWidget, caddr_t, caddr_t);
+extern void setscaley(xcWidget, float *);
+extern void setscalex(xcWidget, float *);
+extern void setorient(xcWidget, short *);
+extern void setpmode(xcWidget, short *);
+extern void setpagesize(xcWidget, XPoint *);
+extern void getdscale(xcWidget, caddr_t, caddr_t);
+extern void getosize(xcWidget, caddr_t, caddr_t);
+extern void getwirewidth(xcWidget, caddr_t, caddr_t);
+extern void getwwidth(xcWidget, caddr_t, caddr_t);
+extern void getfloat(xcWidget, float *, caddr_t);
+extern void setfilename(xcWidget, char **);
+extern void setpagelabel(xcWidget, char *);
+extern void makenewfontbutton();
+extern void newpagemenu(xcWidget, pointertype, caddr_t);
+extern void makenewencodingbutton(char *, char);
+extern void toggle(xcWidget, pointertype, Boolean *);
+extern void inversecolor(xcWidget, pointertype, caddr_t);
+extern void setgridtype(char *);
+extern void renamepage(short);
+extern void renamelib(short);
+extern void setcolormark(int);
+extern void setallstylemarks(u_short);
+extern void setnewcolor(xcWidget, caddr_t);
+extern void addnewcolor(xcWidget, caddr_t, caddr_t);
+extern void setfontmarks(short, short);
 extern void position_popup(xcWidget, xcWidget);
 extern void border_popup(xcWidget, caddr_t, caddr_t);
 extern void color_popup(xcWidget, caddr_t, caddr_t);
 extern void fill_popup(xcWidget, caddr_t, caddr_t);
 extern void param_popup(xcWidget, caddr_t, caddr_t);
-extern Boolean dospecial(void);
+extern void addnewfont(xcWidget, caddr_t, caddr_t);
+#endif
 
 /* from netlist.c: */
 
@@ -571,8 +593,9 @@ extern void NameToPosition(objinstptr, labelptr, XPoint *);
 extern XPoint *NetToPosition(int, objectptr);
 extern void invalidate_netlist(objectptr);
 extern void remove_netlist_element(objectptr, genericptr);
-extern int updatenets(objinstptr);
-extern void createnets(objinstptr);
+extern int updatenets(objinstptr, Boolean);
+extern void createnets(objinstptr, Boolean);
+extern void destroynets();
 extern Boolean nonnetwork(polyptr);
 extern int globalmax();
 extern LabellistPtr geninfolist(objectptr, objinstptr, char *);
@@ -580,6 +603,9 @@ extern void gennetlist(objinstptr);
 extern void gencalls(objectptr);
 extern void search_on_siblings(objinstptr, objinstptr, pushlistptr,
 		short, short, short, short);
+extern char *GetHierarchy(pushlistptr *, Boolean);
+extern Boolean HierNameToObject(objinstptr, char *, pushlistptr *);
+extern void resolve_devindex(objectptr, Boolean);
 extern void copy_bus(Genericlist *, Genericlist *);
 extern Genericlist *is_resolved(genericptr *, pushlistptr, objectptr *);
 extern Boolean highlightnet(objectptr, objinstptr, int, u_char);
@@ -609,11 +635,12 @@ extern void resolve_devnames(objectptr);
 extern void resolve_indices(objectptr, Boolean);
 extern void clear_indices(objectptr);
 extern void unnumber(objectptr);
-extern char *parseinfo(objectptr, CalllistPtr, char *, char *, Boolean);
+extern char *parseinfo(objectptr, objectptr, CalllistPtr, char *, char *, Boolean,
+		Boolean);
 extern int writedevice(FILE *, char *, objectptr, CalllistPtr, char *);
 extern void writeflat(objectptr, CalllistPtr, char *, FILE *, char *);
 extern void writeglobals(objectptr, FILE *);
-extern void writehierarchy(objectptr, CalllistPtr, FILE *, char *);
+extern void writehierarchy(objectptr, objinstptr, CalllistPtr, FILE *, char *);
 extern void writenet(objectptr, char *, char *);
 extern Boolean writepcb(struct Ptab **, objectptr, CalllistPtr, char *, char *);
 extern void outputpcb(struct Ptab *, FILE *);
@@ -629,6 +656,9 @@ extern void destroynets(objectptr);
 extern int  cleartraversed(objectptr);
 extern int  checkvalid(objectptr);
 extern void clearlocalpins(objectptr);
+extern void append_included(char *);
+extern Boolean check_included(char *);
+extern void free_included();
 
 /* from parameter.c: */ 
 
@@ -741,7 +771,6 @@ extern Boolean render_client();
 extern objectptr NameToPageObject(char *, objinstptr *, int *);
 extern objectptr NameToObject(char *, objinstptr *, Boolean);
 extern void callwritenet(xcWidget, pointertype, caddr_t);
-extern void doxschema(xcWidget, pointertype, caddr_t);
 extern void startconnect(xcWidget, caddr_t, caddr_t);
 extern void connectivity(xcWidget, caddr_t, caddr_t);
 extern Boolean setobjecttype(objectptr);
@@ -751,6 +780,7 @@ extern void setsymschem();
 extern int findpageobj(objectptr);
 extern void collectsubschems(int);
 extern int findsubschems(int, objectptr, int, short *);
+extern void copypinlabel(labelptr);
 extern int checkschem(objectptr, char *);
 extern int checksym(objectptr, char *);
 extern void changeotherpins(labelptr, stringpart *);
@@ -765,6 +795,8 @@ extern void xlib_swapschem(xcWidget, pointertype, caddr_t);
 
 /* from selection.c: */
 
+extern void enable_selects(objectptr, short *, int);
+extern void disable_selects(objectptr, short *, int);
 extern void selectfilter(xcWidget, pointertype, caddr_t);
 extern Boolean checkselect(short);
 extern void geneasydraw(short, int, objectptr, objinstptr);
@@ -793,6 +825,7 @@ extern void trackselarea();
 extern Boolean hasparameter(labelptr);
 extern void drawparamlabels(labelptr, short);
 extern stringpart *nextstringpart(stringpart *, objinstptr);
+extern stringpart *nextstringpartrecompute(stringpart *, objinstptr);
 extern stringpart *makesegment(stringpart **, stringpart *);
 extern stringpart *splitstring(int, stringpart **, objinstptr);
 extern stringpart *mergestring(stringpart *);
@@ -831,24 +864,31 @@ extern void fontcat_op(int, int, int);
 
 /* from xcircuit.c: */
 
+extern void Wprintf(char *, ...);
+extern void W1printf(char *, ...);
+extern void W2printf(char *, ...);
+extern void W3printf(char *, ...);
+
+extern XCWindowData *create_new_window();
+extern void delete_window(XCWindowDataPtr);
 extern void printeventmode();
-extern void Wprintf(char *);
 extern void popupprompt(xcWidget, char *, char *, void (*function)(),
                         buttonsave *, const char *);
 extern void getproptext(xcWidget, propstruct *, caddr_t);
 extern int rgb_alloccolor(int, int, int);
 extern void addtocolorlist(xcWidget, int);
-extern void addnewcolorentry(int);
+extern int addnewcolorentry(int);
 extern int xc_getlayoutcolor(int);
+extern int query_named_color(char *);
+extern caddr_t CvtStringToPixel(XrmValuePtr, int *, XrmValuePtr, XrmValuePtr);
 extern void outputpopup(xcWidget, caddr_t, caddr_t);
 extern void docommand();
-extern void W1printf(char *);
-extern void W2printf(char *);
 extern void destroypopup(xcWidget, popupstruct *, caddr_t);
 extern int xc_alloccolor(char *);
 extern void dointr(int);
 extern void DoNothing(xcWidget, caddr_t, caddr_t);
 extern u_short countchanges(char **);
+extern u_short getchanges(objectptr);
 extern void quitcheck(xcWidget, caddr_t, caddr_t);
 extern void quit(xcWidget, caddr_t);
 extern void resizetoolbar();
@@ -861,4 +901,5 @@ extern void updatetext(xcWidget, xcWidgetList, caddr_t);
 extern void delwin(xcWidget, popupstruct *, XClientMessageEvent *);
 #endif
 
+
 extern void makecursors();
