@@ -81,7 +81,7 @@ int string_to_type(const char *);
 extern char _STR2[250], _STR[150];
 extern fontinfo *fonts;
 extern short fontcount;
-extern Clientdata areastruct;
+extern XCWindowData *areawin;
 extern Globaldata xobjs;
 extern short beeper; 
 extern Widget menuwidgets[];
@@ -154,14 +154,14 @@ static PyObject *pypopupprompt(PyObject *self, PyObject *args,
 {
    const char *pfunction = NULL;
    const char *prompt = NULL;
-   buttonsave *savebutton = (buttonsave *)malloc(sizeof(buttonsave));
+   buttonsave *savebutton;
 
    if (!PyArg_ParseTuple(args, "ss:popupprompt", &prompt, &pfunction))
       return PyInt_FromLong(-1L);
 
 
-   getgeneric(savebutton, NULL, NULL, (void *)pfunction);
-   popupprompt(areastruct.area, (char *)prompt, displaystr,
+   savebutton = getgeneric(NULL, NULL, (void *)pfunction);
+   popupprompt(areawin->area, (char *)prompt, displaystr,
 		pypromptvalue, savebutton, dofile);
 
    return PyInt_FromLong(0L);
@@ -234,9 +234,9 @@ static PyObject *xc_new(PyObject *self, PyObject *args)
 	 pathdefaults(*newpath, 0, 0);
 	 newgen = (genericptr *)newpath;
 	 break;
-      case OBJECT:
+      case OBJINST:
          NEW_OBJINST(newobjinst, topobject);
-	 objectdefaults(*newobjinst, NULL, 0, 0);
+	 instancedefaults(*newobjinst, NULL, 0, 0);
 	 newgen = (genericptr *)newobjinst;
 	 break;
       default:
@@ -246,9 +246,7 @@ static PyObject *xc_new(PyObject *self, PyObject *args)
    } 
    topobject->parts++;
    incr_changes(topobject);
-#ifdef SCHEMA
-   topobject->valid = False;
-#endif
+   invalidate_netlist(topobject);
 
    return PyInt_FromLong((long)(*newgen));
 }
@@ -274,7 +272,7 @@ char *type_to_string(int type)
 	 retstr = malloc(12);
 	 strcpy(retstr, "Bezier Curve");
 	 break;
-      case OBJECT:
+      case OBJINST:
 	 retstr = malloc(16);
 	 strcpy(retstr, "Object Instance");
 	 break;
@@ -307,7 +305,7 @@ int string_to_type(const char *etype)
    else if (!strcmp(etype, "Path"))
       return PATH;
    else if (!strcmp(etype, "Object Instance"))
-      return OBJECT;
+      return OBJINST;
    else
       return -1;
 }
@@ -422,7 +420,7 @@ stringpart *PySetStringParts(PyObject *dval)
       if (PyDict_Check(litem)) {
 	 if ((ditem = PyDict_GetItemString(litem, "Parameter")) != NULL) {
 	    newpart->type = PARAM_START;
-	    newpart->data.paramno = (int)PyInt_AsLong(ditem);
+	    newpart->data.string = strdup(PyString_AsString(ditem));
 	 }
 	 else if ((ditem = PyDict_GetItemString(litem, "Font")) != NULL) {
 	    newpart->type = FONT_NAME;
@@ -454,6 +452,7 @@ stringpart *PySetStringParts(PyObject *dval)
 	 string = PyString_AsString(litem);
 	 if (!strcmp(string, "End Parameter")) {
 	    newpart->type = PARAM_END;
+	    newpart->data.string = (u_char *)NULL;
 	 }
 	 else if (!strcmp(string, "Tab Stop")) {
 	    newpart->type = TABSTOP;
@@ -524,7 +523,7 @@ PyObject *PyGetStringParts(stringpart *thisstring)
 	 case PARAM_START:
 	    sdict = PyDict_New();
 	    PyDict_SetItem(sdict, PyString_FromString("Parameter"),
-		  PyInt_FromLong((long)strptr->data.paramno));
+		  PyString_FromString(strptr->data.string));
 	    PyList_SetItem(lstr, i, sdict);
 	    break;
 	 case PARAM_END:
@@ -682,7 +681,7 @@ static PyObject *xc_getpage(PyObject *self, PyObject *args)
 {
    Pagedata *thispage;
    PyObject *rdict, *dlist;
-   int pageno = areastruct.page + 1, i;
+   int pageno = areawin->page + 1, i;
 
    if (!PyArg_ParseTuple(args, "|d:getpage", &pageno))
       return NULL;
@@ -798,9 +797,10 @@ static PyObject *xc_getlibrary(PyObject *self, PyObject *args)
 static PyObject *xc_getobject(PyObject *self, PyObject *args)
 {
    objectptr thisobj = NULL, *libobj;
+   oparamptr ops;
    PyObject *rdict, *dlist, *tpos;
    const char *oname;
-   int i, j, k;
+   int i, j, k, nparam;
 
    if (!PyArg_ParseTuple(args, "s:getobject", &oname))
       return NULL;
@@ -853,18 +853,19 @@ static PyObject *xc_getobject(PyObject *self, PyObject *args)
       PyList_SetItem(dlist, i, PyInt_FromLong((long)(*(thisobj->plist + i))));
    PyDict_SetItem(rdict, PyString_FromString("parts"), dlist);
 
-   if (thisobj->num_params > 0) {
-      dlist = PyList_New(thisobj->num_params);
-      for (i = 0; i < thisobj->num_params; i++) {
-	 if (thisobj->params[i]->type == XC_INT)
+   nparam = get_num_params(thisobj);
+   if (nparam > 0) {
+      dlist = PyList_New(nparam);
+      i = 0;
+      for (ops = thisobj->params; ops != NULL; ops = ops->next) {
+	 i++;
+	 if (ops->type == XC_INT)
+            PyList_SetItem(dlist, i, PyInt_FromLong((long)(ops->parameter.ivalue)));
+	 else if (ops->type == XC_FLOAT)
             PyList_SetItem(dlist, i,
-		PyInt_FromLong((long)(thisobj->params[i]->parameter.ivalue)));
-	 else if (thisobj->params[i]->type == XC_FLOAT)
-            PyList_SetItem(dlist, i,
-		PyFloat_FromDouble((double)(thisobj->params[i]->parameter.fvalue)));
-	 else if (thisobj->params[i]->type == XC_STRING)
-            PyList_SetItem(dlist, i,
-		PyGetStringParts(thisobj->params[i]->parameter.string));
+		PyFloat_FromDouble((double)(ops->parameter.fvalue)));
+	 else if (ops->type == XC_STRING)
+            PyList_SetItem(dlist, i, PyGetStringParts(ops->parameter.string));
       }
       PyDict_SetItem(rdict, PyString_FromString("parameters"), dlist);
    }
@@ -881,12 +882,10 @@ static PyObject *xc_reset()
    return PyInt_FromLong(0L);
 }
 
-#ifdef SCHEMA
 /*--------------------------------------------------------------*/
 /* Get a netlist						*/
 /*--------------------------------------------------------------*/
 
-extern u_int subindex;  /* count for labeling spice subcircuits */
 extern PyObject *pyglobals(objectptr);
 extern PyObject *pytoplevel(objectptr);
 
@@ -894,22 +893,16 @@ static PyObject *xc_netlist()
 {
    PyObject *rdict = NULL;
 
-   if (checkvalid(topobject) == -1) {
-      if (cleartraversed(topobject, 0) == -1)
-         PyErr_SetString(PyExc_TypeError, "Error:  Check circuit for infinite recursion.");
-      else {
-         destroynets();
-         createnets();
-      }
+   if (updatenets(areawin->topinstance, FALSE) <= 0) {
+      PyErr_SetString(PyExc_TypeError, "Error:  Check circuit for infinite recursion.");
+      return NULL;
    }
 
-   subindex = 1;
    rdict = PyDict_New();
    PyDict_SetItem(rdict, PyString_FromString("globals"), pyglobals(topobject));
    PyDict_SetItem(rdict, PyString_FromString("circuit"), pytoplevel(topobject));
    return rdict;
 }
-#endif
 
 /*--------------------------------------------------------------*/
 /* Load the specified file					*/
@@ -921,14 +914,14 @@ static PyObject *xc_load(PyObject *self, PyObject *args)
    int pageno = 0, savepage;
 
    if (!PyArg_ParseTuple(args, "s|i:load", &filename, &pageno))
-      return PyInt_FromLong((long)areastruct.page);
+      return PyInt_FromLong((long)areawin->page);
 
-   if (--pageno < 0) pageno = areastruct.page;
-   savepage = areastruct.page;
+   if (--pageno < 0) pageno = areawin->page;
+   savepage = areawin->page;
 
    strcpy(_STR2, filename);
    if (savepage != pageno) newpage(pageno);
-   startloadfile();
+   startloadfile(-1);
    if (savepage != pageno) newpage(savepage);
 
    return PyString_FromString(filename);
@@ -943,7 +936,7 @@ static PyObject *xc_page(PyObject *self, PyObject *args)
    int pageno;
 
    if (!PyArg_ParseTuple(args, "i:page", &pageno))
-      return PyInt_FromLong((long)areastruct.page);
+      return PyInt_FromLong((long)areawin->page);
 
    newpage(pageno - 1);
    return PyInt_FromLong((long)pageno);
@@ -965,18 +958,18 @@ static PyObject *xc_zoom(PyObject *self, PyObject *args)
       return NULL;
    }
    
-   save = areastruct.zoomfactor;
+   save = areawin->zoomfactor;
 
    if (factor < 1.0) {
-      areastruct.zoomfactor = 1.0 / factor;
-      zoomout(NULL, NULL, NULL);
+      areawin->zoomfactor = 1.0 / factor;
+      zoomout(0, 0);	/* Needs to be fixed---give x, y for drag fn */
    }
    else {
-      areastruct.zoomfactor = factor;
-      zoomin(NULL, NULL, NULL);
+      areawin->zoomfactor = factor;
+      zoomin(0, 0);	/* Needs to be fixed---see above */
    }
 
-   areastruct.zoomfactor = save;
+   areawin->zoomfactor = save;
    return PyFloat_FromDouble((double)topobject->viewscale);
 }
 
@@ -988,7 +981,6 @@ static PyObject *xc_pan(PyObject *self, PyObject *args)
 {
    PyObject *cornerpos, *xobj, *yobj = NULL, *pval;
    int x, y;
-   XButtonEvent bevent;
    XPoint upoint, wpoint;
 
    if (!PyArg_ParseTuple(args, "O|O:pan", &xobj, &yobj))
@@ -1009,13 +1001,11 @@ static PyObject *xc_pan(PyObject *self, PyObject *args)
    }
 
    user_to_window(upoint, &wpoint);
-   bevent.x = wpoint.x;
-   bevent.y = wpoint.y;
-   panbutton((u_int)5, &bevent);
+   panbutton((u_int)5, wpoint.x, wpoint.y, 0.33);  /* fixed fraction */
 
    cornerpos = PyTuple_New(2);
-   PyTuple_SetItem(cornerpos, 0, PyInt_FromLong((long)areastruct.pcorner->x));
-   PyTuple_SetItem(cornerpos, 1, PyInt_FromLong((long)areastruct.pcorner->y));
+   PyTuple_SetItem(cornerpos, 0, PyInt_FromLong((long)areawin->pcorner.x));
+   PyTuple_SetItem(cornerpos, 1, PyInt_FromLong((long)areawin->pcorner.y));
 
    return cornerpos;
 }
@@ -1029,8 +1019,8 @@ static PyObject *xc_getwindow()
    PyObject *windowsize;
 
    windowsize = PyTuple_New(2);
-   PyTuple_SetItem(windowsize, 0, PyInt_FromLong((long)areastruct.width));
-   PyTuple_SetItem(windowsize, 1, PyInt_FromLong((long)areastruct.height));
+   PyTuple_SetItem(windowsize, 0, PyInt_FromLong((long)areawin->width));
+   PyTuple_SetItem(windowsize, 1, PyInt_FromLong((long)areawin->height));
 
    return windowsize;
 }
@@ -1093,7 +1083,7 @@ static PyObject *xc_getattr(PyObject *self, PyObject *args)
    if (lstr != NULL)
       PyDict_SetItem(rdict, PyString_FromString("color"), lstr);
 
-   switch((*gelem)->type) {
+   switch(ELEMENTTYPE(*gelem)) {
       case LABEL:
 	 PyDict_SetItem(rdict, PyString_FromString("position"), 
 		make_pair(&(TOLABEL(gelem)->position)));
@@ -1103,10 +1093,8 @@ static PyObject *xc_getattr(PyObject *self, PyObject *args)
 		PyFloat_FromDouble((double)TOLABEL(gelem)->scale));
 	 PyDict_SetItem(rdict, PyString_FromString("justify"),
 		PyInt_FromLong((long)TOLABEL(gelem)->justify));
-#ifdef SCHEMA
 	 PyDict_SetItem(rdict, PyString_FromString("pin"),
 		PyInt_FromLong((long)TOLABEL(gelem)->pin));
-#endif
 	 lstr = PyGetStringParts(TOLABEL(gelem)->string);
 
 	 PyDict_SetItem(rdict, PyString_FromString("string"), lstr);
@@ -1161,7 +1149,7 @@ static PyObject *xc_getattr(PyObject *self, PyObject *args)
 	 }
 	 PyDict_SetItem(rdict, PyString_FromString("parts"), dlist);
 	 break;
-      case OBJECT:
+      case OBJINST:
 	 PyDict_SetItem(rdict, PyString_FromString("position"), 
 		make_pair(&(TOOBJINST(gelem)->position)));
 	 PyDict_SetItem(rdict, PyString_FromString("rotation"),
@@ -1203,7 +1191,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
    if ((dval = PyDict_GetItemString(attrdict, "type")) != NULL) {
       int dtype = string_to_type(PyString_AsString(dval));
       if (dtype < 0) return NULL;
-      if (dtype != (*gelem)->type) {
+      if (dtype != ELEMENTTYPE(*gelem)) {
          PyErr_SetString(PyExc_TypeError,
 		"Attempt to change the type of an object.");
          return NULL;
@@ -1221,7 +1209,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
       if (PyTuple_Check(dval) && PyTuple_Size(dval) == 2) {
          if ((pval = PyTuple_GetItem(dval, 0)) != NULL) {
             short xpos = (short)PyInt_AsLong(pval);
-            switch((*gelem)->type) {
+            switch(ELEMENTTYPE(*gelem)) {
 	       case ARC:
 	          TOARC(gelem)->position.x = xpos;
 	          calcarc(TOARC(gelem));
@@ -1229,7 +1217,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 	       case LABEL:
 	          TOLABEL(gelem)->position.x = xpos;
 	          break;
-	       case OBJECT:
+	       case OBJINST:
 	          TOOBJINST(gelem)->position.x = xpos;
 	          break;
                default:
@@ -1240,7 +1228,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 
          if ((pval = PyTuple_GetItem(dval, 1)) != NULL) {
             short ypos = (short)PyInt_AsLong(pval);
-            switch((*gelem)->type) {
+            switch(ELEMENTTYPE(*gelem)) {
 	       case ARC:
 	          TOARC(gelem)->position.y = ypos;
 	          calcarc(TOARC(gelem));
@@ -1248,7 +1236,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 	       case LABEL:
 	          TOLABEL(gelem)->position.y = ypos;
 	          break;
-	       case OBJECT:
+	       case OBJINST:
 	          TOOBJINST(gelem)->position.y = ypos;
 	          break;
                default:
@@ -1265,7 +1253,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 
    if ((dval = PyDict_GetItemString(attrdict, "style")) != NULL) {
       short dstyle = (short)PyInt_AsLong(dval);
-      switch((*gelem)->type) {
+      switch(ELEMENTTYPE(*gelem)) {
          case POLYGON:
 	    TOPOLY(gelem)->style = dstyle;
 	    break;
@@ -1286,7 +1274,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 
    if ((dval = PyDict_GetItemString(attrdict, "linewidth")) != NULL) {
       float dwidth = (float)PyFloat_AsDouble(dval);
-      switch((*gelem)->type) {
+      switch(ELEMENTTYPE(*gelem)) {
          case POLYGON:
 	    TOPOLY(gelem)->width = dwidth;
 	    break;
@@ -1307,11 +1295,11 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 
    if ((dval = PyDict_GetItemString(attrdict, "scale")) != NULL) {
       float dscale = (float)PyFloat_AsDouble(dval);
-      switch((*gelem)->type) {
+      switch(ELEMENTTYPE(*gelem)) {
          case LABEL:
 	    TOLABEL(gelem)->scale = dscale;
 	    break;
-         case OBJECT:
+         case OBJINST:
 	    TOOBJINST(gelem)->scale = dscale;
 	    break;
          default:
@@ -1322,11 +1310,11 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 
    if ((dval = PyDict_GetItemString(attrdict, "rotation")) != NULL) {
       float drot = (float)PyFloat_AsDouble(dval);
-      switch((*gelem)->type) {
+      switch(ELEMENTTYPE(*gelem)) {
          case LABEL:
 	    TOLABEL(gelem)->rotation = drot;
 	    break;
-         case OBJECT:
+         case OBJINST:
 	    TOOBJINST(gelem)->rotation = drot;
 	    break;
          default:
@@ -1337,7 +1325,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 
    /* Dictionary entries specific to certain xcircuit types */
 
-   switch((*gelem)->type) {
+   switch(ELEMENTTYPE(*gelem)) {
       case LABEL:
 	 if ((dval = PyDict_GetItemString(attrdict, "justify")) != NULL) {
 	    TOLABEL(gelem)->justify = PyInt_AsLong(dval);
@@ -1346,11 +1334,9 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 	    freelabel(TOLABEL(gelem)->string);
 	    TOLABEL(gelem)->string = PySetStringParts(dval);
 	 }
-#ifdef SCHEMA
 	 if ((dval = PyDict_GetItemString(attrdict, "pin")) != NULL) {
 	    TOLABEL(gelem)->pin = PyInt_AsLong(dval);
 	 }
-#endif
 	 break;
       case ARC:
 	 if ((dval = PyDict_GetItemString(attrdict, "start angle")) != NULL) {
@@ -1424,7 +1410,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 	 break;
    }
 
-   calcbbox(areastruct.topinstance);
+   calcbbox(areawin->topinstance);
    return PyInt_FromLong((long)(*gelem));
 }
 /*--------------------------------------------------------------*/
@@ -1437,7 +1423,7 @@ static PyObject *xc_set(PyObject *self, PyObject *args)
    const char *sarg1, *sarg2;
    int i;
    Boolean a = True; 
-   short cpage = areastruct.page;
+   short cpage = areawin->page;
 
    if (!PyArg_ParseTuple(args, "ss:set", &sarg1, &sarg2))
       return NULL;
@@ -1458,26 +1444,26 @@ static PyObject *xc_set(PyObject *self, PyObject *args)
 
       for (i = 0; i < fontcount; i++) {
 	 if (!strcmp(fonts[i].psname, sarg2)) {
-            areastruct.psfont = i;
+            areawin->psfont = i;
             setdefaultfontmarks();
 	    break;
 	 }
       }
    }
    else if (!strcmp(sarg1, "fontscale")) {
-      sscanf(sarg2, "%f", &areastruct.textscale);
+      sscanf(sarg2, "%f", &areawin->textscale);
    }
    else if (!strcmp(sarg1, "axis") || !strcmp(sarg1, "axes")) {
-      areastruct.axeson = a;
-      toggle(GridAxesButton, &areastruct.axeson, NULL);
+      areawin->axeson = a;
+      toggle(GridAxesButton, (pointertype)&areawin->axeson, NULL);
    }
    else if (!strcmp(sarg1, "grid")) {
-      areastruct.gridon = a;
-      toggle(GridGridButton, &areastruct.gridon, NULL);
+      areawin->gridon = a;
+      toggle(GridGridButton, (pointertype)&areawin->gridon, NULL);
    }  
    else if (!strcmp(sarg1, "snap") || !strcmp(sarg1, "snap-to")) {
-      areastruct.snapto = a;
-      toggle(SnaptoSnaptoButton, &areastruct.snapto, NULL);
+      areawin->snapto = a;
+      toggle(SnaptoSnaptoButton, (pointertype)&areawin->snapto, NULL);
    }
    else if (!strcmp(sarg1, "gridspace")) {
       sscanf(sarg2, "%f", &xobjs.pagelist[cpage]->gridspace);
@@ -1499,12 +1485,12 @@ static PyObject *xc_set(PyObject *self, PyObject *args)
       else if (!strcmp(sarg2, "normal")) boxedit(NULL, NORMAL, NULL);
    }
    else if (!strcmp(sarg1, "linewidth")) {
-      sscanf(sarg2, "%f", &areastruct.linewidth);
+      sscanf(sarg2, "%f", &areawin->linewidth);
    }
    else if (!strcmp(sarg1, "colorscheme")) {
       if (!strcmp(sarg2, "inverse"))
-         areastruct.invert = False;
-      inversecolor(NULL, &areastruct.invert, NULL);
+         areawin->invert = False;
+      inversecolor(NULL, (pointertype)&areawin->invert, NULL);
    }
    else if (!strcmp(sarg1, "coordstyle")) {
       if (!strcmp(sarg2, "cm") || !strcmp(sarg2, "centimeters")) {
@@ -1521,15 +1507,12 @@ static PyObject *xc_set(PyObject *self, PyObject *args)
          xobjs.pagelist[cpage]->orient = 0;  /* Portrait */
    }
 
-#ifdef SCHEMA
    else if (!strcmp(sarg1, "xschema") || !strcmp(sarg1, "schema")) {
-      areastruct.schemon = a;
-      doxschema(OptionsEnableXSchemaButton, 1, NULL);
+      /* Do nothing---retained for backward compatibility only */
    }
-#endif
 #ifdef HAVE_XPM
    else if (!strcmp(sarg1, "toolbar")) {
-      areastruct.toolbar_on = a;
+      areawin->toolbar_on = a;
       dotoolbar(OptionsDisableToolbarButton, NULL, NULL);
     }
 #endif
@@ -1576,7 +1559,7 @@ static PyObject *xc_library(PyObject *self, PyObject *args)
    }
 
    if (libnum >= xobjs.numlibs || libnum < 0)
-      libnum = createlibrary();
+      libnum = createlibrary(FALSE);
    else
       libnum += LIBRARY - 1;
 
@@ -1668,12 +1651,15 @@ static PyObject *xc_bind(PyObject *self, PyObject *args)
 	 keywstate = -1;
 	 func = string_to_func(keyname, NULL);
       }
-      binding = binding_to_string(keywstate, func);
+      if (keywstate == -1)
+         binding = function_binding_to_string(0, func);
+      else
+         binding = key_binding_to_string(0, keywstate);
       retobj = PyString_FromString(binding);
       free(binding);
    }
    else {
-      if (add_keybinding(keyname, function) < 0) {
+      if (add_keybinding(0, keyname, function) < 0) {
 	 /* Function may be a Python function, not a C function */
          keywstate = string_to_key(keyname);
 	 sprintf(_STR2, "keydict[%d] = %s\n", keywstate, function);
@@ -1702,7 +1688,7 @@ static PyObject *xc_unbind(PyObject *self, PyObject *args)
       flags |= KEYOVERRIDE;
    }
 
-   remove_keybinding(keyname, function);
+   remove_keybinding(areawin->area, keyname, function);
    return PyString_FromString(keyname);
 }
 
@@ -1734,7 +1720,7 @@ static PyObject *xc_refresh(PyObject *self, PyObject *args)
       
    refresh(NULL, NULL, NULL);
 
-   while(XCheckWindowEvent(dpy, areastruct.areawin, ~NoEventMask, &event))
+   while (XCheckWindowEvent(dpy, areawin->window, ~NoEventMask, &event))
          XtDispatchEvent(&event);
 
    return PyInt_FromLong(0L);
@@ -1834,8 +1820,7 @@ static PyObject *xc_newtool(PyObject *self, PyObject *args)
       return NULL;
 
    if (XpmReadFileToImage(dpy, (char *)pixfile, &iret, NULL, &attr) < 0) {
-      sprintf(_STR, "Cannot find or open pixmap file \'%s\'", pixfile);
-      Wprintf(_STR);
+      Wprintf("Cannot find or open pixmap file \'%s\'", pixfile);
       XpmCreateImageFromData(dpy, q_xpm, &iret, NULL, &attr);
    }
 
@@ -1958,9 +1943,7 @@ static PyMethodDef xc_methods[] = {
 #ifdef HAVE_XPM
    {"newtool",		xc_newtool,	1},
 #endif
-#ifdef SCHEMA
    {"netlist",		xc_netlist,	1},
-#endif
    {NULL,	NULL}			/* sentinel */
 };
 
@@ -2024,8 +2007,7 @@ void defaultscript()
    if ((fd = fopen(_STR2, "r")) == NULL) {
       sprintf(_STR2, "%s/%s", BUILTINS_DIR, STARTUP_FILE);
       if ((fd = fopen(_STR2, "r")) == NULL) {
-         sprintf(_STR, "Failed to open startup script \"%s\"\n", STARTUP_FILE);
-         Wprintf(_STR);
+         Wprintf("Failed to open startup script \"%s\"\n", STARTUP_FILE);
 	 return;
       }
    }
@@ -2042,14 +2024,13 @@ void execscript()
 
    flags = 0;
 
-   xc_tilde_expand(_STR2);
+   xc_tilde_expand(_STR2, 249);
    if ((fd = fopen(_STR2, "r")) != NULL) {
       PyRun_SimpleFile(fd, _STR2);
       refresh(NULL, NULL, NULL);
    }
    else {
-      sprintf(_STR, "Failed to open script file \"%s\"\n", _STR2);
-      Wprintf(_STR);
+      Wprintf("Failed to open script file \"%s\"\n", _STR2);
    }  
 }
 
@@ -2071,7 +2052,7 @@ void loadrcfile()
       
    /* try first in current directory, then look in user's home directory */
       
-   xc_tilde_expand(_STR2);
+   xc_tilde_expand(_STR2, 249);
    if ((fd = fopen(_STR2, "r")) == NULL) {
       if (userdir != NULL) {
          sprintf(_STR2, "%s/%s", userdir, USER_RC_FILE);
@@ -2085,15 +2066,15 @@ void loadrcfile()
 
    if (!(flags & FONTOVERRIDE)) {
       loadfontfile("Helvetica");
-      if (areastruct.psfont == -1)
+      if (areawin->psfont == -1)
          for (i = 0; i < fontcount; i++)
 	    if (!strcmp(fonts[i].psname, "Helvetica")) {
-	       areastruct.psfont = i;
+	       areawin->psfont = i;
 	       break;
 	    }
    }
 
-   if (areastruct.psfont == -1) areastruct.psfont = 0;
+   if (areawin->psfont == -1) areawin->psfont = 0;
 
    setdefaultfontmarks();
       
@@ -2120,14 +2101,13 @@ void loadrcfile()
       addnewcolorentry(xc_alloccolor("Brown"));
    }  
 
-#ifdef SCHEMA
    /* These colors must be enabled whether or not colors are overridden, */
    /* because they are needed by the schematic capture system.		 */
 
    addnewcolorentry(xc_getlayoutcolor(LOCALPINCOLOR));
    addnewcolorentry(xc_getlayoutcolor(GLOBALPINCOLOR));
    addnewcolorentry(xc_getlayoutcolor(INFOLABELCOLOR));
-#endif
+   addnewcolorentry(xc_getlayoutcolor(RATSNESTCOLOR));
    addnewcolorentry(xc_getlayoutcolor(BBOXCOLOR));
 
    if (!(flags & KEYOVERRIDE))
