@@ -17,10 +17,12 @@
 #ifndef XC_WIN32
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+#endif
 
 #ifdef TCL_WRAPPER 
 #include <tk.h>
 #else
+#ifndef XC_WIN32
 #include "Xw/Xw.h"
 #endif
 #endif
@@ -45,7 +47,7 @@
 extern char _STR2[250], _STR[150];
 extern fontinfo *fonts;
 extern short fontcount;
-extern Clientdata areastruct;
+extern XCWindowData *areawin;
 extern Globaldata xobjs;
 extern short beeper;
 extern int *appcolors;
@@ -70,9 +72,9 @@ short execcommand(short pflags, char *cmdptr)
    Boolean a;
    char type[50], value[50];
    char *argptr;
-   short cpage = areastruct.page;
+   short cpage = areawin->page;
 
-   /* Note:  areastruct.page is set to 0 before loading the .xcircuitrc	*/
+   /* Note:  areawin->page is set to 0 before loading the .xcircuitrc	*/
    /*file, so .xcircuitrc file commands act on page 0 only;  from the	*/
    /* command line they act on the current page.			*/
 
@@ -93,24 +95,23 @@ short execcommand(short pflags, char *cmdptr)
    else if (!(a = strcmp(cmdptr, "enable")) || !strcmp(cmdptr, "disable")) {
       sscanf(argptr, "%149s", _STR);
       if (!strncmp(_STR, "grid", 4)) {
-	 areastruct.gridon = a ? True : False;
-	 toggle(GridGridButton, &areastruct.gridon, NULL);
+	 areawin->gridon = a ? True : False;
+	 toggle(GridGridButton, (pointertype)&areawin->gridon, NULL);
       }
       else if (!strncmp(_STR, "snap", 4) || !strncmp(_STR, "snap-to", 4)) {
-	 areastruct.snapto = a ? True : False;
-	 toggle(SnaptoSnaptoButton, &areastruct.snapto, NULL);
+	 areawin->snapto = a ? True : False;
+	 toggle(SnaptoSnaptoButton, (pointertype)&areawin->snapto, NULL);
       }
       else if (!strcmp(_STR, "axis") || !strcmp(_STR, "axes")) {
-	 areastruct.axeson = a ? True : False;
-	 toggle(GridAxesButton, &areastruct.axeson, NULL);
+	 areawin->axeson = a ? True : False;
+	 toggle(GridAxesButton, (pointertype)&areawin->axeson, NULL);
       }
       else if (!strcmp(_STR, "xschema") || !strcmp(_STR, "schema")) {
-	 areastruct.schemon = a ? True : False;
-	 doxschema(OptionsEnableXSchemaButton, 1, NULL);
+	 /* Do nothing---retained for backward compatibility only */
       }
 #ifdef HAVE_XPM
       else if (!strcmp(_STR, "toolbar")) {
-	 areastruct.toolbar_on = a ? True : False;
+	 areawin->toolbar_on = a ? True : False;
 	 dotoolbar(OptionsDisableToolbarButton, NULL, NULL);
       }
 #endif
@@ -145,11 +146,11 @@ short execcommand(short pflags, char *cmdptr)
    /* load extra files */
 
    else if (!strcmp(cmdptr, "load")) {
-      int pageno = areastruct.page;
-      int savepage = areastruct.page;
+      int pageno = areawin->page;
+      int savepage = areawin->page;
       if (sscanf(argptr, "%249s %d", _STR, &pageno) >= 1) {
 	 if (pageno != savepage) newpage(pageno);
-	 startloadfile();
+	 startloadfile(-1);
 	 if (pageno != savepage) newpage(savepage);
       }
       else
@@ -221,8 +222,11 @@ short execcommand(short pflags, char *cmdptr)
 	    keywstate = -1;
 	    func = string_to_func(_STR, NULL);
 	 }
-	 binding = binding_to_string(keywstate, func);
-	 Wprintf(binding);
+	 if (keywstate == -1)
+	    binding = function_binding_to_string(0, func);
+	 else
+	    binding = key_binding_to_string(0, keywstate);
+	 Wprintf("%s", binding);
 	 free(binding);
       }
       else if (args == 0)
@@ -230,9 +234,8 @@ short execcommand(short pflags, char *cmdptr)
       else {
 	 /* Use entire line after 1st argument, so multiple words are allowed */
 	 sptr = argptr + strlen(_STR);
-         if (add_keybinding(_STR, sptr) < 0) {
-	    sprintf(_STR, "Function \"%s\" is not known\n", sptr);
-	    Wprintf(_STR);
+         if (add_keybinding(areawin->area, _STR, sptr) < 0) {
+	    Wprintf("Function \"%s\" is not known\n", sptr);
 	 }
       }
    }
@@ -245,7 +248,7 @@ short execcommand(short pflags, char *cmdptr)
          flags |= KEYOVERRIDE;
       }
       sscanf(argptr, "%149s %249s", _STR, _STR2);
-      remove_keybinding(_STR, _STR2);
+      remove_keybinding(areawin->area, _STR, _STR2);
    }
 
    /* beep mode */
@@ -270,7 +273,7 @@ short execcommand(short pflags, char *cmdptr)
 
       refresh(NULL, NULL, NULL);
 
-      while(XCheckWindowEvent(dpy, areastruct.areawin, ~NoEventMask, &event))
+      while(XCheckWindowEvent(dpy, areawin->window, ~NoEventMask, &event))
          XtDispatchEvent(&event);
    }
 
@@ -290,7 +293,7 @@ short execcommand(short pflags, char *cmdptr)
       if (!strncmp(argptr, "font", 4)) {
 	 short i;
 	 if (strstr(argptr + 4, "scale")) {
-	    sscanf(argptr + 4, "%*s %f", &areastruct.textscale);
+	    sscanf(argptr + 4, "%*s %f", &areawin->textscale);
 	 }
 	 else {
 	    sscanf(argptr, "%*s %49s", value);
@@ -300,7 +303,7 @@ short execcommand(short pflags, char *cmdptr)
 	    if (i == fontcount) {
 	       loadfontfile(value);
 	    }
-   	    areastruct.psfont = i;
+   	    areawin->psfont = i;
 	    setdefaultfontmarks();
 	 }
       }
@@ -314,14 +317,14 @@ short execcommand(short pflags, char *cmdptr)
       }
       else if (!strncmp(argptr, "line", 4)) {
 	 if (strstr(argptr + 4, "width")) {
-	    sscanf(argptr + 4, "%*s %f", &areastruct.linewidth);
+	    sscanf(argptr + 4, "%*s %f", &areawin->linewidth);
 	 }
       }
       else if (!strncmp(argptr, "colorscheme", 11)) {
 	 sscanf(argptr, "%*s %49s", value);
 	 if (!strcmp(value, "inverse") || !strcmp(value, "2"))
-	    areastruct.invert = False;
-	 inversecolor(NULL, &areastruct.invert, NULL);
+	    areawin->invert = False;
+	 inversecolor(NULL, (pointertype)&areawin->invert, NULL);
       }
       else if (!strncmp(argptr, "coordstyle", 10)) {
 	 sscanf(argptr, "%*s %49s", value);
@@ -418,8 +421,7 @@ void defaultscript()
    if ((fd = fopen(_STR2, "r")) == NULL) {
       sprintf(_STR2, "%s/%s", BUILTINS_DIR, STARTUP_FILE);
       if ((fd = fopen(_STR2, "r")) == NULL) {
-         sprintf(_STR, "Failed to open startup script \"%s\"\n", STARTUP_FILE);
-         Wprintf(_STR);
+         Wprintf("Failed to open startup script \"%s\"\n", STARTUP_FILE);
 	 return;
       }
    }
@@ -434,11 +436,10 @@ void execscript()
 {
    FILE *fd;
 
-   xc_tilde_expand(_STR2);
+   xc_tilde_expand(_STR2, 249);
    if ((fd = fopen(_STR2, "r")) != NULL) readcommand(0, fd);
    else {
-      sprintf(_STR, "Failed to open script file \"%s\"\n", _STR2);
-      Wprintf(_STR);
+      Wprintf("Failed to open script file \"%s\"\n", _STR2);
    }
 }
 
@@ -456,7 +457,7 @@ void loadrcfile()
 
    /* try first in current directory, then look in user's home directory */
 
-   xc_tilde_expand(_STR2);
+   xc_tilde_expand(_STR2, 249);
    if ((fd = fopen(_STR2, "r")) == NULL) {
       if (userdir != NULL) {
          sprintf(_STR2, "%s/%s", userdir, USER_RC_FILE);
@@ -470,7 +471,7 @@ void loadrcfile()
    if (!(flags & FONTOVERRIDE)) {
       loadfontfile("Helvetica");
       for (i = 0; i < fontcount; i++)
-	 if (!strcmp(fonts[i].psname, "Helvetica")) areastruct.psfont = i;
+	 if (!strcmp(fonts[i].psname, "Helvetica")) areawin->psfont = i;
    }
    setdefaultfontmarks();
 
