@@ -52,12 +52,6 @@ typedef struct {
    short yaxis;
 } arcinfo;
 
-typedef struct {
-   short number;
-   genericptr *element;
-   short *idx;
-} uselection;
-
 typedef struct {
    genericptr element;		/* element being edited */
    union {
@@ -67,13 +61,37 @@ typedef struct {
    } save;
 } editelement;
 
+typedef struct {
+   genericptr element;		/* element modified */
+   float scale;			/* old scale value */
+} scaleinfo;
+
 /*----------------------------------------------------------------------*/
 /* Externally declared variables					*/
 /*----------------------------------------------------------------------*/
 
 extern Globaldata xobjs;
-extern Clientdata areastruct;
-extern short eventmode;
+extern XCWindowData *areawin;
+
+/*----------------------------------------------------------------------*/
+/* Attempt to set the window.  If the window exists, set it as current	*/
+/* and return TRUE.  Otherwise, return FALSE.  This prevents the undo	*/
+/* mechanism from crashing if we close a window and then try to undo	*/
+/* events that were created relative to it.				*/
+/*----------------------------------------------------------------------*/
+
+Boolean setwindow(XCWindowData *trywindow)
+{
+   XCWindowData *chkwin;
+
+   for (chkwin = xobjs.windowlist; chkwin != NULL; chkwin = chkwin->next) {
+      if (chkwin == trywindow) {
+	 areawin = trywindow;
+	 return TRUE;
+      }
+   }
+   return FALSE;
+}
 
 /*----------------------------------------------------------------------*/
 /* remember_selection ---						*/
@@ -107,7 +125,7 @@ uselection *remember_selection(objinstptr topinst, short *slist, int number)
 }
 
 /*----------------------------------------------------------------------*/
-/* Create a selection list in areastruct from the saved uselection	*/
+/* Create a selection list in areawin from the saved uselection	*/
 /* record.								*/
 /*----------------------------------------------------------------------*/
 
@@ -252,8 +270,8 @@ int select_previous(Undoptr thisrecord)
 	 case XCF_Copy:
 	 case XCF_Select:
 	    srec = (uselection *)chkrecord->undodata;
-	    areastruct.selectlist = regen_selection(thisrecord->thisinst, srec);
-	    areastruct.selects = (areastruct.selectlist) ? srec->number : 0;
+	    areawin->selectlist = regen_selection(thisrecord->thisinst, srec);
+	    areawin->selects = (areawin->selectlist) ? srec->number : 0;
 	    return 0;
       }
    }
@@ -330,15 +348,17 @@ void undelete_one_element(objinstptr thisinst, genericptr thiselem)
 void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 {
    va_list args;
-   int usize, drawmode, npage, opage, *idata, snum, deltax, deltay, dir;
+   int usize, drawmode, nval, oval, *idata, snum, deltax, deltay, dir;
    short *slist;
    objectptr delobj;
    objinstptr newinst;
    Undoptr newrecord;
    uselection *srec;
    editelement *erec;
+   scaleinfo *escale;
    genericptr egen, newgen;
    XPoint *fpoint;
+   double scale;
 
    /* Do not register new events while running undo/redo actions! */
    if (eventmode == UNDO_MODE) return;
@@ -352,6 +372,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
    newrecord->last = NULL;
    newrecord->type = type;
    newrecord->thisinst =  thisinst;
+   newrecord->window = areawin;
    newrecord->undodata = (char *)NULL;
    newrecord->idata = 0;
 
@@ -394,14 +415,14 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 
       case XCF_Page:
 	 /* 2 args:							*/
-	 /*	opage = original page (int)				*/
-	 /*	npage = new page (int)					*/
-	 opage = va_arg(args, int);
-	 npage = va_arg(args, int);
+	 /*	oval = original integer value				*/
+	 /*	nval = new integer value				*/
+	 oval = va_arg(args, int);
+	 nval = va_arg(args, int);
 	 idata = (int *)malloc(sizeof(int));
-	 *idata = npage;
+	 *idata = nval;
 	 newrecord->undodata = (char *)idata;
-	 newrecord->idata = opage;
+	 newrecord->idata = oval;
 	 break;
 
       case XCF_Pop:
@@ -449,7 +470,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 	    case LABEL:
 	       erec->save.string =
 			stringcopyall(((labelptr)egen)->string,
-			areastruct.topinstance);
+			areawin->topinstance);
 	       newrecord->idata = ((labelptr)egen)->justify;
 	       break;
 	    case POLYGON:
@@ -472,6 +493,32 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 	 newrecord->undodata = (char *)erec;
 	 break;
 
+      case XCF_ChangeStyle:
+      case XCF_Justify:
+	 /* 2 args:							*/
+	 /*	egen = element that was changed (with new value)	*/
+	 /*	oval = old style value					*/
+	 egen = va_arg(args, genericptr);
+	 oval = va_arg(args, int);
+	 newrecord->undodata = (char *)egen;
+	 newrecord->idata = oval;
+	 break;
+
+      case XCF_Scale:
+	 /* 2 args:							*/
+	 /*	egen = element that was changed (with new value)	*/
+	 /*	ofloat = old float value				*/
+
+	 egen = va_arg(args, genericptr);
+	 scale = va_arg(args, double); 		/* warning! only takes "double"! */
+
+	 escale = (scaleinfo *)malloc(sizeof(scaleinfo));
+	 escale->element = egen;
+	 escale->scale = (float)scale;
+
+	 newrecord->undodata = (char *)escale;
+	 break;
+
       case XCF_Flip_X: case XCF_Flip_Y:
 	 /* 1 arg:							*/
 	 /*	fpoint = point of flip (XPoint *)			*/
@@ -522,10 +569,19 @@ short undo_one_action()
    objinstptr thisinst;
    uselection *srec;
    editelement *erec;
+   scaleinfo *escale;
    short *slist;
-   XPoint *delta;
+   XPoint *delta, position;
    int i, snum, npage, opage;
+   float fnum;
    genericptr egen;
+   labelptr thislabel;
+   pathptr thispath;
+   polyptr thispoly;
+   arcptr thisarc;
+   splineptr thisspline;
+   graphicptr thisgraphic;
+   XCWindowData *savewindow = areawin;
 
    /* Undo the recorded action and shift the undo record pointer.	*/
 
@@ -538,6 +594,13 @@ short undo_one_action()
    xobjs.undostack = thisrecord->next;
    xobjs.redostack = thisrecord;
 
+   /* Set window, if event occurred in a different window */
+   if (setwindow(thisrecord->window) == FALSE) {
+      Wprintf("Error:  Undo event in nonexistant window!  Flushing stack.\n");
+      flush_undo_stack();
+      return 0;
+   }
+
    /* Setting eventmode to UNDO_MODE prevents register_for_undo() from 	*/
    /* being called again while executing the event.			*/
 
@@ -549,12 +612,12 @@ short undo_one_action()
       case XCF_Delete:
 	 unselect_all();
 	 thisobj = (objectptr)thisrecord->undodata;
-	 areastruct.selects = thisobj->parts;
-	 areastruct.selectlist = xc_undelete(thisrecord->thisinst,
+	 areawin->selects = thisobj->parts;
+	 areawin->selectlist = xc_undelete(thisrecord->thisinst,
 			thisobj, (short)thisrecord->idata,
 			recover_selectlist(thisrecord));
-	 srec = remember_selection(thisrecord->thisinst, areastruct.selectlist,
-			areastruct.selects);
+	 srec = remember_selection(thisrecord->thisinst, areawin->selectlist,
+			areawin->selects);
 	 thisrecord->undodata = (char *)srec;
 	 draw_all_selected();
 	 break;
@@ -579,18 +642,18 @@ short undo_one_action()
 	    delete_one_element(thisrecord->thisinst, egen);
 
 	 /* Put back all the parts */
-	 areastruct.selects = thisobj->parts;
-	 areastruct.selectlist = xc_undelete(thisrecord->thisinst,
+	 areawin->selects = thisobj->parts;
+	 areawin->selectlist = xc_undelete(thisrecord->thisinst,
 			thisobj, (short)thisrecord->idata,
 			recover_selectlist(thisrecord));
-	 srec = remember_selection(thisrecord->thisinst, areastruct.selectlist,
-			areastruct.selects);
+	 srec = remember_selection(thisrecord->thisinst, areawin->selectlist,
+			areawin->selects);
 	 thisrecord->undodata = (char *)srec;
 	 draw_all_selected();
 	 break;
 
       case XCF_Push:
-	 popobject(areastruct.area, 0, NULL);
+	 popobject(areawin->area, 0, NULL);
 	 break;
 
       case XCF_Pop:
@@ -601,6 +664,14 @@ short undo_one_action()
 	 newpage(thisrecord->idata);
 	 break;
 
+      case XCF_Justify:
+	 thislabel = (labelptr)(thisrecord->undodata);
+	 snum = thisrecord->idata;
+	 thisrecord->idata = thislabel->justify;
+	 thislabel->justify = snum;
+	 drawarea(areawin->area, NULL, NULL);
+	 break;
+
       case XCF_Select:
 
 	 /* If there was a previous selection in the undo list,	*/
@@ -621,7 +692,7 @@ short undo_one_action()
 	 }
 	 else {
 	    delete_one_element(thisrecord->thisinst, egen);
-	    drawarea(areastruct.area, NULL, NULL);
+	    drawarea(areawin->area, NULL, NULL);
 	 }
 	 break;
 
@@ -653,7 +724,7 @@ short undo_one_action()
 	       epoly->number = thisrecord->idata;
 	       erec->save.points = tmppts;
 	       thisrecord->idata = tmpnum;
-	       drawarea(areastruct.area, NULL, NULL);
+	       drawarea(areawin->area, NULL, NULL);
 	    } break;
 	    case SPLINE: {
 	       pointlist tmppts;
@@ -664,7 +735,7 @@ short undo_one_action()
 	       free(erec->save.points);
 	       erec->save.points = tmppts;
 	       calcspline(espline);
-	       drawarea(areastruct.area, NULL, NULL);
+	       drawarea(areawin->area, NULL, NULL);
 	    } break;
 	    case ARC: {
 	       arcinfo tmpinfo;
@@ -679,7 +750,7 @@ short undo_one_action()
 	       earc->yaxis = erec->save.arcspecs->yaxis;
 	       *(erec->save.arcspecs) = tmpinfo;
 	       calcarc(earc);
-	       drawarea(areastruct.area, NULL, NULL);
+	       drawarea(areawin->area, NULL, NULL);
 	    }
 	 }
 	 break;
@@ -703,28 +774,97 @@ short undo_one_action()
 
 	 /* Revert selection to previously selected */
 	 select_previous(thisrecord);
-	 drawarea(areastruct.area, NULL, NULL);
+	 drawarea(areawin->area, NULL, NULL);
+	 break;
+
+      case XCF_ChangeStyle:
+	 /* Style changes */
+	 egen = (genericptr)thisrecord->undodata;
+	 snum = thisrecord->idata;
+	 switch(egen->type) {
+	    case PATH:
+	       thispath = (pathptr)(thisrecord->undodata);
+	       thisrecord->idata = thispath->style;
+	       thispath->style = snum;
+	       break;
+	    case POLYGON:
+	       thispoly = (polyptr)(thisrecord->undodata);
+	       thisrecord->idata = thispoly->style;
+	       thispoly->style = snum;
+	       break;
+	    case ARC:
+	       thisarc = (arcptr)(thisrecord->undodata);
+	       thisrecord->idata = thisarc->style;
+	       thisarc->style = snum;
+	       break;
+	    case SPLINE:
+	       thisspline = (splineptr)(thisrecord->undodata);
+	       thisrecord->idata = thisspline->style;
+	       thisspline->style = snum;
+	       break;
+	 }
+	 drawarea(areawin->area, NULL, NULL);
+	 break;
+
+      case XCF_Scale:
+	 escale = (scaleinfo *)thisrecord->undodata;
+	 egen = escale->element;
+	 fnum = escale->scale;
+	 switch(egen->type) {
+	    case PATH:
+	       thispath = (pathptr)egen;
+	       escale->scale = thispath->width;
+	       thispath->width = fnum;
+	       break;
+	    case POLYGON:
+	       thispoly = (polyptr)egen;
+	       escale->scale = thispoly->width;
+	       thispoly->width = fnum;
+	       break;
+	    case ARC:
+	       thisarc = (arcptr)egen;
+	       escale->scale = thisarc->width;
+	       thisarc->width = fnum;
+	       break;
+	    case SPLINE:
+	       thisspline = (splineptr)egen;
+	       escale->scale = thisspline->width;
+	       thisspline->width = fnum;
+	       break;
+	    case OBJINST:
+	       thisinst = (objinstptr)egen;
+	       escale->scale = thisinst->scale;
+	       thisinst->scale = fnum;
+	       break;
+	    case GRAPHIC:
+	       thisgraphic = (graphicptr)egen;
+	       escale->scale = thisgraphic->scale;
+	       thisgraphic->scale = fnum;
+	       thisgraphic->valid = FALSE;
+	       break;
+	 }
+	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
       case XCF_Flip_X:
-	 areastruct.save = *((XPoint *)thisrecord->undodata);
-	 elementflip();
+	 position = *((XPoint *)thisrecord->undodata);
+	 elementflip(&position);
 	 break;
 
       case XCF_Flip_Y:
-	 areastruct.save = *((XPoint *)thisrecord->undodata);
-	 elementvflip();
+	 position = *((XPoint *)thisrecord->undodata);
+	 elementvflip(&position);
 	 break;
 
       case XCF_Rotate:
-	 areastruct.save = *((XPoint *)thisrecord->undodata);
-	 elementrotate(-thisrecord->idata);
+	 position = *((XPoint *)thisrecord->undodata);
+	 elementrotate(-thisrecord->idata, &position);
 	 break;
 
       case XCF_Move:
 	 delta = (XPoint *)thisrecord->undodata;
 	 placeselects(-(delta->x), -(delta->y), NULL);
-	 drawarea(areastruct.area, NULL, NULL);
+	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
       default:
@@ -739,6 +879,7 @@ short undo_one_action()
       Fprintf(stderr, "Warning:  Unfinished undo series in stack!\n");
       thisrecord->idx = -thisrecord->idx;
    }
+   areawin = savewindow;
    return thisrecord->idx;
 }
 
@@ -765,10 +906,20 @@ short redo_one_action()
    objectptr thisobj;
    genericptr egen;
    short *slist;
-   XPoint *delta;
+   XPoint *delta, position;
    uselection *srec;
    editelement *erec;
+   scaleinfo *escale;
    int i, tpage, snum;
+   float fnum;
+   labelptr thislabel;
+   arcptr thisarc;
+   pathptr thispath;
+   splineptr thisspline;
+   polyptr thispoly;
+   graphicptr thisgraphic;
+   objinstptr thisinst;
+   XCWindowData *savewindow = areawin;
 
    /* Undo the recorded action and shift the undo record pointer.	*/
 
@@ -780,6 +931,13 @@ short redo_one_action()
    xobjs.undostack = thisrecord;
    xobjs.redostack = thisrecord->last;
 
+   /* Set window, if event occurred in a different window */
+   if (setwindow(thisrecord->window) == FALSE) {
+      Wprintf("Error:  Undo event in nonexistant window!  Flushing stack.\n");
+      flush_undo_stack();
+      return 0;
+   }
+
    eventmode = UNDO_MODE;
 
    /* type-dependent part */
@@ -794,7 +952,7 @@ short redo_one_action()
 	 thisrecord->undodata = (char *)thisobj;
 	 thisrecord->idata = (int)DRAW;
 	 unselect_all();
-	 drawarea(areastruct.area, NULL, NULL);
+	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
       /* Unfinished! */
@@ -807,8 +965,16 @@ short redo_one_action()
 	 newpage(*((int *)thisrecord->undodata));
 	 break;
 
+      case XCF_Justify:
+	 thislabel = (labelptr)(thisrecord->undodata);
+	 snum = thisrecord->idata;
+	 thisrecord->idata = thislabel->justify;
+	 thislabel->justify = snum;
+	 drawarea(areawin->area, NULL, NULL);
+	 break;
+
       case XCF_Pop:
-	 popobject(areastruct.area, 0, NULL);
+	 popobject(areawin->area, 0, NULL);
 	 break;
 
       case XCF_Push:
@@ -818,8 +984,8 @@ short redo_one_action()
       case XCF_Select:
 	 unselect_all();
 	 srec = (uselection *)thisrecord->undodata;
-	 areastruct.selectlist = regen_selection(thisrecord->thisinst, srec);
-	 areastruct.selects = (areastruct.selectlist) ? srec->number : 0;
+	 areawin->selectlist = regen_selection(thisrecord->thisinst, srec);
+	 areawin->selects = (areawin->selectlist) ? srec->number : 0;
 	 draw_all_selected();
 	 break;
 
@@ -839,11 +1005,11 @@ short redo_one_action()
 	 thisobj = (objectptr)thisrecord->undodata;
 	 if (thisobj != NULL) {
 	    unselect_all();
-	    areastruct.selects = thisobj->parts;
-	    areastruct.selectlist = xc_undelete(thisrecord->thisinst, thisobj, DRAW,
+	    areawin->selects = thisobj->parts;
+	    areawin->selectlist = xc_undelete(thisrecord->thisinst, thisobj, DRAW,
 			NULL);
 	    thisrecord->undodata = (char *)remember_selection(thisrecord->thisinst,
-			areastruct.selectlist, areastruct.selects);
+			areawin->selectlist, areawin->selects);
 	    draw_all_selected();
 	 }
 	 break;
@@ -853,7 +1019,7 @@ short redo_one_action()
       case XCF_Spline: case XCF_Dot: case XCF_Graphic:
 	 egen = (genericptr)thisrecord->undodata;
 	 undelete_one_element(thisrecord->thisinst, egen);
-	 drawarea(areastruct.area, NULL, NULL);
+	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
       case XCF_Edit:
@@ -884,7 +1050,7 @@ short redo_one_action()
 	       epoly->number = thisrecord->idata;
 	       erec->save.points = tmppts;
 	       thisrecord->idata = tmpnum;
-	       drawarea(areastruct.area, NULL, NULL);
+	       drawarea(areawin->area, NULL, NULL);
 	    } break;
 	    case SPLINE: {
 	       pointlist tmppts;
@@ -895,7 +1061,7 @@ short redo_one_action()
 	       free(erec->save.points);
 	       erec->save.points = tmppts;
 	       calcspline(espline);
-	       drawarea(areastruct.area, NULL, NULL);
+	       drawarea(areawin->area, NULL, NULL);
 	    } break;
 	    case ARC: {
 	       arcinfo tmpinfo;
@@ -910,30 +1076,99 @@ short redo_one_action()
 	       earc->yaxis = erec->save.arcspecs->yaxis;
 	       *(erec->save.arcspecs) = tmpinfo;
 	       calcarc(earc);
-	       drawarea(areastruct.area, NULL, NULL);
+	       drawarea(areawin->area, NULL, NULL);
 	    }
 	 }
 	 break;
 
       case XCF_Flip_X:
-	 areastruct.save = *((XPoint *)thisrecord->undodata);
-	 elementflip();
+	 position = *((XPoint *)thisrecord->undodata);
+	 elementflip(&position);
 	 break;
 
       case XCF_Flip_Y:
-	 areastruct.save = *((XPoint *)thisrecord->undodata);
-	 elementvflip();
+	 position = *((XPoint *)thisrecord->undodata);
+	 elementvflip(&position);
+	 break;
+
+      case XCF_ChangeStyle:
+	 /* Style changes */
+	 egen = (genericptr)thisrecord->undodata;
+	 snum = thisrecord->idata;
+	 switch(egen->type) {
+	    case PATH:
+	       thispath = (pathptr)(thisrecord->undodata);
+	       thisrecord->idata = thispath->style;
+	       thispath->style = snum;
+	       break;
+	    case POLYGON:
+	       thispoly = (polyptr)(thisrecord->undodata);
+	       thisrecord->idata = thispoly->style;
+	       thispoly->style = snum;
+	       break;
+	    case ARC:
+	       thisarc = (arcptr)(thisrecord->undodata);
+	       thisrecord->idata = thisarc->style;
+	       thisarc->style = snum;
+	       break;
+	    case SPLINE:
+	       thisspline = (splineptr)(thisrecord->undodata);
+	       thisrecord->idata = thisspline->style;
+	       thisspline->style = snum;
+	       break;
+	 }
+	 drawarea(areawin->area, NULL, NULL);
+	 break;
+
+      case XCF_Scale:
+	 escale = (scaleinfo *)thisrecord->undodata;
+	 egen = escale->element;
+	 fnum = escale->scale;
+	 switch(egen->type) {
+	    case PATH:
+	       thispath = (pathptr)egen;
+	       escale->scale = thispath->width;
+	       thispath->width = fnum;
+	       break;
+	    case POLYGON:
+	       thispoly = (polyptr)egen;
+	       escale->scale = thispoly->width;
+	       thispoly->width = fnum;
+	       break;
+	    case ARC:
+	       thisarc = (arcptr)egen;
+	       escale->scale = thisarc->width;
+	       thisarc->width = fnum;
+	       break;
+	    case SPLINE:
+	       thisspline = (splineptr)egen;
+	       escale->scale = thisspline->width;
+	       thisspline->width = fnum;
+	       break;
+	    case OBJINST:
+	       thisinst = (objinstptr)egen;
+	       escale->scale = thisinst->scale;
+	       thisinst->scale = fnum;
+	       break;
+	    case GRAPHIC:
+	       thisgraphic = (graphicptr)egen;
+	       escale->scale = thisgraphic->scale;
+	       thisgraphic->scale = fnum;
+	       thisgraphic->valid = FALSE;
+	       break;
+	 }
+	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
       case XCF_Rotate:
-	 areastruct.save = *((XPoint *)thisrecord->undodata);
-	 elementrotate(thisrecord->idata);
+	 position = *((XPoint *)thisrecord->undodata);
+	 elementrotate(thisrecord->idata, &position);
 	 break;
 
       case XCF_Move:
 	 delta = (XPoint *)thisrecord->undodata;
 	 placeselects(delta->x, delta->y, NULL);
-	 drawarea(areastruct.area, NULL, NULL);
+	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
       default:
@@ -942,6 +1177,8 @@ short redo_one_action()
    }
 
    eventmode = NORMAL_MODE;	/* Set on a per-event-type basis? */
+   areawin = savewindow;
+
    return thisrecord->idx;
 }
 
@@ -1062,6 +1299,8 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
 	 break;
 
       case XCF_Push:
+      case XCF_ChangeStyle:
+      case XCF_Justify:
 	 /* Do nothing --- undodata points to a valid element */
 	 break;
 
