@@ -31,7 +31,6 @@ typedef unsigned long	u_long;
 #define DrawLine	XDrawLine
 #define DrawLines	XDrawLines
 #define DrawPoint	XDrawPoint
-#define DrawPoints	XDrawPoints
 #define FillPolygon	XFillPolygon
 
 #define SetForeground	XSetForeground
@@ -67,6 +66,8 @@ typedef unsigned long	u_long;
 
 #ifdef TCL_WRAPPER
   #define malloc Tcl_Alloc
+  /* (see definition of my_calloc in the asg subdirectory) */
+  // #define calloc(a,b) Tcl_Alloc(a * b)
   #define free(a) Tcl_Free((char *)(a))
   #define realloc(a,b) Tcl_Realloc((char *)(a), b)
   #undef strdup
@@ -137,15 +138,16 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 
 /* Names used for convenience throughout the source code */
 
-#define topobject	areastruct.topinstance->thisobject
-#define hierobject	areastruct.hierstack->thisinst->thisobject
+#define eventmode	areawin->event_mode
+#define topobject	areawin->topinstance->thisobject
+#define hierobject	areawin->hierstack->thisinst->thisobject
 
 #define PLIST_INCR(a) \
 	(a)->plist = (genericptr *) realloc ((a)->plist, \
 	((a)->parts + 1) * sizeof(genericptr))
 
 #define ENDPART		topobject->plist + topobject->parts
-#define EDITPART	topobject->plist + areastruct.editpart
+#define EDITPART	topobject->plist + areawin->editpart
 
 /* Type checks (argument "a" is type genericptr) */
 
@@ -174,7 +176,7 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 
 /* conversions from a selection to a specific type */
 
-#define SELTOGENERICPTR(a) ((areastruct.hierstack == NULL) ? \
+#define SELTOGENERICPTR(a) ((areawin->hierstack == NULL) ? \
 			(topobject->plist + *(a)) : \
 			(hierobject->plist + *(a)))
 
@@ -317,7 +319,8 @@ enum editmode {
   EARC_MODE,		/* Editing an existing arc */
   ESPLINE_MODE,		/* Editing an existing spline */
   EPATH_MODE,		/* Editing an existing path */
-  ASSOC_MODE		/* Choosing an associated schematic or symbol */
+  ASSOC_MODE,		/* Choosing an associated schematic or symbol */
+  CATMOVE_MODE		/* Moving objects in or between libraries */
 };
 
 /*----------------------------------------------------------------------*/
@@ -327,12 +330,6 @@ enum editmode {
 enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #ifdef ASG
 	IMPORTSPICE,
-#endif
-#ifdef USE_WIN32_COM
-	COMSCRIPT,
-#ifdef USE_WIN32_DOTNET
-	COMDOTNET,
-#endif
 #endif
 	LOAD_MODES
 };
@@ -378,6 +375,19 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define PARAM_START	17  /* bounds a parameter; data = param key	*/
 #define PARAM_END	18  /* bounds a parameter; no data 		*/
 
+/* Actions translated to keystates (numbering continues from above) */
+
+#define TEXT_RETURN	19
+#define TEXT_HOME	20
+#define TEXT_END	21
+#define TEXT_SPLIT	22
+#define TEXT_DOWN	23
+#define TEXT_UP		24
+#define TEXT_LEFT	25
+#define TEXT_RIGHT	26
+#define TEXT_DELETE	27
+#define TEXT_DEL_PARAM	28
+
 #define SPECIAL		63  /* used only when called from menu		*/
 #define NULL_TYPE	255 /* used as a placeholder			*/
 
@@ -466,6 +476,7 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define TRIVIAL 2		/* Symbol as non-schematic element */
 #define SYMBOL 3		/* Symbol associated with a schematic */
 #define FUNDAMENTAL 4		/* Standalone symbol */
+#define NONETWORK 5		/* Do not netlist this object */
 
 #define LOCAL 1
 #define GLOBAL 2
@@ -514,10 +525,10 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define WAITFOR		appcursors[9]
 #define HAND		appcursors[10]
 
-#define DEFAULTCURSOR	(*areastruct.defaultcursor)
+#define DEFAULTCURSOR	(*areawin->defaultcursor)
 
 /*----------------------------------------------------------------------*/
-/* selection-mechanism structures					*/
+/* integer and floating-point coordinate list structures		*/
 /*----------------------------------------------------------------------*/
 
 typedef XPoint* pointlist;
@@ -617,6 +628,16 @@ typedef struct {
    eparamptr	passed;
 } generic, *genericptr;		/* (convenience function for retypecasting) */
 
+/*----------------------------------------------------------------------*/
+/* selection-mechanism structures					*/
+/*----------------------------------------------------------------------*/
+
+typedef struct {
+   short number;
+   genericptr *element;
+   short *idx;
+} uselection;
+
 /*----------------------------------------------------------------------*/
 /* Bounding box								*/
 /*----------------------------------------------------------------------*/
@@ -795,7 +816,7 @@ typedef struct _selection {
 #define select_element(a) recurse_select_element(a, UNDO_MORE) 
 #define select_add_element(a) recurse_select_element(a, UNDO_DONE) 
 
-#define easydraw(a, b)	 geneasydraw(a, b, topobject, areastruct.topinstance)
+#define easydraw(a, b)	 geneasydraw(a, b, topobject, areawin->topinstance)
 
 /*----------------------------------------------------------------------*/
 /* Netlist structures for schematic capture				*/
@@ -963,7 +984,7 @@ typedef struct _matrix {
 /* Some convenience functions for matrix manipulation			*/
 /*----------------------------------------------------------------------*/
 
-#define DCTM  areastruct.MatStack
+#define DCTM  areawin->MatStack
 
 /*----------------------------------------------------------------------*/
 /* button tap/press definitions 					*/
@@ -1008,7 +1029,7 @@ typedef struct _alias {
 #define xcScreen			Tk_Screen
 #define xcParent			Tk_Parent
 #define xcDispatchEvent(a)		Tk_HandleEvent(a)
-#define xcAddTimeOut(a, b, c, d)	Tcl_CreateTimerHandler(b, c, d)
+#define xcAddTimeOut(a, b, c, d)	Tcl_CreateTimerHandler((int)(b), c, d)
 #define xcRemoveTimeOut			Tcl_DeleteTimerHandler
 #define xcTimeOutProc			Tcl_TimerProc *
 #define xcIntervalId			Tcl_TimerToken
@@ -1086,21 +1107,18 @@ typedef struct {
    /* color scheme 1 */
    Pixel	fg, bg;
    Pixel	gridpix, snappix, selectpix, axespix;
-   Pixel	buttonpix, querypix, filterpix;
-   Pixel	auxpix, barpix, parampix;
+   Pixel	buttonpix, filterpix, auxpix, barpix, parampix;
 
    /* color scheme 2 */
-
    Pixel	fg2, bg2;
    Pixel	gridpix2, snappix2, selectpix2, axespix2;
-   Pixel	buttonpix2, querypix2, filterpix2;
-   Pixel	auxpix2, barpix2, parampix2;
+   Pixel	buttonpix2, auxpix2, parampix2;
 
    int		width, height, timeout;
-   XFontStruct	*helpfont, *filefont;
+   XFontStruct	*filefont;
 
 #ifndef TCL_WRAPPER
-   XFontStruct	*xcfont, *textfont, *titlefont;
+   XFontStruct	*xcfont, *textfont, *titlefont, *helpfont;
 #endif
 
 } ApplicationData, *ApplicationDataPtr;
@@ -1110,19 +1128,19 @@ typedef struct {
 /*----------------------------------------------------------------------*/
 
 #define XTopSetForeground(a) if (a == DEFAULTCOLOR) SetForeground(dpy, \
-	areastruct.gc, FOREGROUND); else SetForeground(dpy, areastruct.gc, a)
+	areawin->gc, FOREGROUND); else SetForeground(dpy, areawin->gc, a)
 
 #define XSetXORFg(a,b) if (a == DEFAULTCOLOR) SetForeground(dpy, \
-	areastruct.gc, FOREGROUND ^ b); else SetForeground(dpy, areastruct.gc,\
+	areawin->gc, FOREGROUND ^ b); else SetForeground(dpy, areawin->gc,\
 	a ^ b)
 
-#define XcSetFunction(z) SetFunction(dpy, areastruct.gc, z); \
-	areastruct.gctype = z
+#define XcSetFunction(z) SetFunction(dpy, areawin->gc, z); \
+	areawin->gctype = z
 
-#define XcSetForeground(z) XTopSetForeground(z); areastruct.gccolor = \
+#define XcSetForeground(z) XTopSetForeground(z); areawin->gccolor = \
 	((z) == DEFAULTCOLOR) ? FOREGROUND : (z)
 
-#define XcSetXORFg(y,z) XSetXORFg(y,z); areastruct.gccolor = \
+#define XcSetXORFg(y,z) XSetXORFg(y,z); areawin->gccolor = \
 	((y) == DEFAULTCOLOR) ? (FOREGROUND ^ z) : (y ^ z)
 
 /*----------------------------------------------------------------------*/
@@ -1170,6 +1188,7 @@ typedef struct {
 typedef struct _keybinding *keybindingptr;
 
 typedef struct _keybinding {
+   xcWidget window;		/* per-window function, or NULL */
    int keywstate;
    int function;
    short value;
@@ -1186,7 +1205,7 @@ enum {
    XCF_ENDDATA = -2,		XCF_SPACER  /* -1 */,
    XCF_Page	 /* 0 */,	XCF_Justify /* 1 */,
    XCF_Superscript /* 2 */,	XCF_Subscript /* 3 */,
-   XCF_Normalscript /* 4 */,	XCF_Nextfont /* 5 */,
+   XCF_Normalscript /* 4 */,	XCF_Font /* 5 */,
    XCF_Boldfont /* 6 */,	XCF_Italicfont /* 7 */,
    XCF_Normalfont /* 8 */,	XCF_Underline /* 9 */,
    XCF_Overline /* 10 */,	XCF_ISO_Encoding /* 11 */,
@@ -1238,7 +1257,8 @@ enum {
    XCF_Finish /* 102 */,	XCF_Cancel_Last /* 103 */,
    XCF_Sim /* 104 */,		XCF_SPICE /* 105 */,
    XCF_PCB /* 106 */,		XCF_SPICEflat /* 107 */,
-   XCF_Text_Delete_Param /* 108 */,
+   XCF_Color /* 108 */,		XCF_Text_Delete_Param /* 109 */,
+   XCF_Scale /* 110 */,		XCF_ChangeStyle /* 111 */,
    NUM_FUNCTIONS
 };
 
@@ -1279,12 +1299,16 @@ typedef struct {
 /* all the critical data needed by the drawing window 			*/
 /*----------------------------------------------------------------------*/
 
-typedef struct {
+typedef struct _windowdata *XCWindowDataPtr;
+
+typedef struct _windowdata {
+
+   XCWindowDataPtr next;	/* next window in list */
 
    /* widgets and X11 parameters */
    xcWidget	area;
    xcWidget	scrollbarh, scrollbarv;
-   Window	areawin;
+   Window	window;
    GC 		gc;
    int		gccolor, gctype;
    xcIntervalId time_id;
@@ -1292,8 +1316,8 @@ typedef struct {
    /* global page parameters */
    short	width, height;
    short	page;
-   float	*vscale;	/* pointer to proper scale */
-   XPoint	*pcorner;	/* pointer to page position */
+   float	vscale;		/* proper scale */
+   XPoint	pcorner;	/* page position */
 
    /* global option defaults */
    float	textscale;
@@ -1302,8 +1326,8 @@ typedef struct {
    short	psfont;
    short	justify;
    u_short	style;
-   short	filter;		/* selection filter */
    int		color;
+   short	filter;		/* selection filter */
    Boolean	manhatn;
    Boolean	boxedit;
    Boolean	snapto;
@@ -1313,11 +1337,10 @@ typedef struct {
    Boolean	axeson;
    Boolean	invert;
    Boolean	mapped;		/* indicates if window is drawable */
-   Boolean	filefilter;	/* Is the file list filtered? */
    char		buschar;	/* Character indicating vector notation */
    Boolean	editinplace;
-   Boolean	schemon;
    Boolean	pinpointon;
+   Boolean	attachto;
 #ifdef HAVE_XPM
    Boolean	toolbar_on;
 #endif
@@ -1332,11 +1355,12 @@ typedef struct {
    Matrixptr	MatStack;
    pushlistptr	stack;
    pushlistptr	hierstack;
+   short	event_mode;
    short	editcycle;
    short	editpart, editsubpart;
    char		*lastbackground;
    Cursor	*defaultcursor;
-} Clientdata;
+} XCWindowData;
 
 /* Record for undo function */ 
 
@@ -1348,6 +1372,7 @@ typedef struct _undostack {
    u_int type;		/* type of event */
    short idx;		/* counter for undo event */
    objinstptr thisinst;	/* instance of object in which event occurred */
+   XCWindowData *window;/* window in which event occurred */
    int idata;		/* simple undedicated integer datum */
    char *undodata;	/* free space to be malloc'd */
 			/* (size dependent on "type") */
@@ -1361,22 +1386,32 @@ typedef struct _undostack {
 
 typedef struct {
    char		*filename;	/* Last file loaded into the library */
+   char		flags;		/* Flags for library page (changed, read-only) */
    short	number;
    objectptr	*library;
    liblistptr	instlist;	/* List of instances */
 } Library;
 
+/* Known flags for library pages */
+
+#define LIBRARY_CHANGED		0x01
+#define LIBRARY_READONLY	0x02
+
 /*----------------------------------------------------------------------*/
 /* A convenient structure for holding all the object lists		*/
 /*----------------------------------------------------------------------*/
 
 typedef struct {
+   char		*libsearchpath;	 /* list of directories to search */
+   char		*filesearchpath; /* list of directories to search */
    char		*tempfile;
    char		*tempdir;
    Boolean	retain_backup;
    xcIntervalId	timeout_id;
    int		save_interval;
    short	do_replace;
+   Boolean	filefilter;	/* Is the file list filtered? */
+   Boolean	hold;		/* allow HOLD modifiers on buttons */
    u_short	new_changes;
    short	numlibs;
    short	pages;
@@ -1388,6 +1423,7 @@ typedef struct {
    objinstptr   *libtop;
    Imagedata	*imagelist;
    short	images;
+   XCWindowData *windowlist;	/* linked list of known windows */
 } Globaldata;
 
 /*----------------------------------------------------------------------*/
@@ -1427,4 +1463,3 @@ typedef struct _toolbarstruct {
 /* Menus and Toolbars are taken care of entirely by scripts in the Tcl/	*/
 /* Tk version of xcircuit.						*/
 /*----------------------------------------------------------------------*/
-
